@@ -7,19 +7,13 @@
 
 #if !defined(VMS) && !defined(MSDOS) && defined(CRYPT_PATH)
 
-#include <curses.h>
-
-#if defined(BSD42) || defined(BSD43)
 #include <sys/types.h>
 #include <sys/file.h>
-#else
 #include <fcntl.h>
-#endif
-
+#include <unistd.h>
+#include "compat.h"
 #include "sc.h"
 
-char        *getpass();
-
 int         Crypt = 0;
 #define MAXKEYWORDSIZE 30
 char	    KeyWord[MAXKEYWORDSIZE] = {""};
@@ -47,7 +41,7 @@ creadfile(char *save, int  eraseflg)
     }
 
     deraw(1);
-    (void) strcpy(KeyWord, getpass("Enter key:"));
+    strlcpy(KeyWord, getpass("Enter key:"), sizeof KeyWord);
     goraw();
 
     if ((pid=fork()) == 0) {		/* if child		 */
@@ -83,7 +77,7 @@ creadfile(char *save, int  eraseflg)
     while (pid != wait(&fildes)) /**/;
     linelim = -1;
     if (eraseflg) {
-	(void) strcpy(curfile, save);
+	strlcpy(curfile, save, sizeof curfile);
 	modflg = 0;
     }
 }
@@ -110,14 +104,12 @@ cwritefile(char *fname, int r0, int c0, int rn, int cn)
 	return (-1);
 	}
 
-    (void) strcpy(save, fname);
+    strlcpy(save, fname, sizeof save);
 
     busave = findhome(save);
-#ifdef DOBACKUPS
-    if (!backup_file(busave) &&
+    if (dobackups && !backup_file(busave) &&
 	    (yn_ask("Could not create backup copy, Save anyway?: (y,n)") != 1))
 	return (0);
-#endif
     if ((fildes = open (busave, O_TRUNC|O_WRONLY|O_CREAT, 0600)) < 0) {
 	error("Can't create file \"%s\"", save);
 	return (-1);
@@ -130,7 +122,7 @@ cwritefile(char *fname, int r0, int c0, int rn, int cn)
 
     if (KeyWord[0] == '\0') {
 	deraw(1);
-	(void) strcpy(KeyWord, getpass("Enter key:"));
+	strlcpy(KeyWord, getpass("Enter key:"), sizeof KeyWord);
 	goraw();
     }
 
@@ -163,7 +155,7 @@ cwritefile(char *fname, int r0, int c0, int rn, int cn)
     (void) fclose(f);
     (void) close(pipefd[1]);
     while (pid != wait(&fildes)) /**/;
-    (void) strcpy(curfile,save);
+    strlcpy(curfile, save, sizeof curfile);
 
     modflg = 0;
     error("File \"%s\" written (encrypted).", curfile);
