@@ -93,11 +93,10 @@
 #include <string.h>
 #include <sys/types.h>
 #include <time.h>
+#include <limits.h>
+#include "compat.h"
 #include "sc.h"
 
-#define bool	int
-#define true	1
-#define false	0
 #define EOS	'\0'
 #define MAXBUF	256
 
@@ -112,7 +111,7 @@ char *colformat[COLFORMATS];
 /*****************************************************************************/
 
 bool
-format(char *fmt, int lprecision, double val, char *buf, int buflen)
+format(char *fmt, int lprecision, double val, char *buf, size_t buflen)
 {
     register char *cp;
     char *tmp, *tp;
@@ -137,7 +136,8 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
 	tmpfmt2 = scxrealloc(tmpfmt2, fmtlen);
 	exptmp = scxrealloc(exptmp, fmtlen);
     }
-    fmt = strcpy(tmpfmt1, fmt);
+    strlcpy(tmpfmt1, fmt, fmtlen);
+    fmt = tmpfmt1;
     if (buflen + 1 > mantlen) {
     	mantlen = buflen + 40;
 	mantissa = scxrealloc(mantissa, mantlen);
@@ -210,7 +210,8 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
 	    cp++;
 	else if (*cp == 'e' || *cp == 'E') {
 	    if (cp[1] == '+' || cp[1] == '-') {
-		exponent = strcpy(exptmp, cp);
+		strlcpy(exptmp, cp, fmtlen);
+		exponent = exptmp;
 		*cp = EOS;
 		if (val != 0.0) {
 		    while (val < 1.0) {
@@ -256,8 +257,8 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
 	}
 	zero_pad = strlen(decimal) - zero_pad;
     }
-    (void) sprintf(prtfmt, "%%.%dlf", width);
-    (void) sprintf(mantissa, prtfmt, val);
+    snprintf(prtfmt, sizeof prtfmt, "%%.%dlf", width);
+    snprintf(mantissa, mantlen, prtfmt, val);
     for (cp = integer = mantissa; *cp != dpoint && *cp != EOS; cp++) {
 	if (*integer == '0')
 	    integer++;
@@ -282,7 +283,7 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
     static	char *citmp = NULL, *cftmp = NULL;
     static	unsigned cilen = 0, cflen = 0;
     char *ci, *cf, *ce;
-    int len_ci, len_cf, len_ce;
+    unsigned int len_ci, len_cf, len_ce;
     bool ret = false;
     
     ci = fmt_int(integer, fmt, comma, negative);
@@ -291,7 +292,8 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
     	cilen = len_ci + 40;
 	citmp = scxrealloc(citmp, cilen);
     }
-    ci = strcpy(citmp, ci);
+    strlcpy(citmp, ci, cilen);
+    ci = citmp;
 
     cf = decimal ? fmt_frac(fraction, decimal, lprecision) : "";
     len_cf = strlen(cf);
@@ -299,7 +301,8 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
     	cflen = len_cf + 40;
 	cftmp = scxrealloc(cftmp, cilen);
     }
-    cf = strcpy(cftmp, cf);
+    strlcpy(cftmp, cf, cilen);
+    cf = cftmp;
 
     ce = (exponent) ? fmt_exp(exp_val, exponent) : "";
     len_ce = strlen(ce);
@@ -308,7 +311,7 @@ format(char *fmt, int lprecision, double val, char *buf, int buflen)
  *   ce = strcpy(scxmalloc((unsigned)((len_ce = strlen(ce)) + 1)), ce);
  */
     if (len_ci + len_cf + len_ce < buflen) {
-	(void) sprintf(buf, "%s%s%s", ci, cf, ce);
+	snprintf(buf, buflen, "%s%s%s", ci, cf, ce);
 	ret = true;
     }
 
@@ -345,7 +348,7 @@ fmt_int(char *val,	/* integer part of the value to be formatted */
  * format the value
  */
     f = strlen(fmt) - 1;
-    v = (digit >= 0) ? strlen(val) - 1 : -1;
+    v = (digit >= 0) ? (ssize_t)strlen(val) - 1 : -1;
     while (f >= 0 || v >= 0) {
 	if (f > 0 && fmt[f-1] == '\\') {
 	    *bufptr++ = fmt[f--];
@@ -431,9 +434,9 @@ fmt_exp(int val,	/* value of the exponent */
 	val = -val;
 	negative = false;
     }
-    (void) sprintf(valbuf, "%d", val);
+    snprintf(valbuf, sizeof valbuf, "%d", val);
   
-    (void) strcat(buf, fmt_int(valbuf, fmt, false, negative));
+    strlcat(buf, fmt_int(valbuf, fmt, false, negative), sizeof buf);
     return (buf);
 }
 
@@ -507,12 +510,12 @@ engformat(int fmt, int width, int lprecision, double val, char *buf, int buflen)
     if (fmt >= 0 && fmt < COLFORMATS && colformat[fmt])
 	return (format(colformat[fmt], lprecision, val, buf, buflen));
     if (fmt == REFMTFIX)
-	(void) sprintf(buf,"%*.*f", width, lprecision, val);
+	snprintf(buf, buflen, "%*.*f", width, lprecision, val);
     if (fmt == REFMTFLT)
-	(void) sprintf(buf,"%*.*E", width, lprecision, val);
+	snprintf(buf, buflen, "%*.*E", width, lprecision, val);
     if (fmt == REFMTENG) {
 	if (val == 0e0) {	/* Hack to get zeroes to line up in engr fmt */
-	    (void) sprintf((buf-1),"%*.*f ", width, lprecision, val);
+	    snprintf(buf - 1, buflen, "%*.*f ", width, lprecision, val);
 	} else {
 	    engabs = (val);
 	    if ( engabs <  0e0)       engabs = -engabs;
@@ -531,11 +534,11 @@ engformat(int fmt, int width, int lprecision, double val, char *buf, int buflen)
 	    if ((engabs >= 1e18)  && (engabs <  1e21 )) engind=12;
 	    if ((engabs < 1e-18)  || (engabs >= 1e21 )) {
 		/* Revert to floating point */
-		(void) sprintf(buf,"%*.*E", width, lprecision, val);
+		snprintf(buf, buflen, "%*.*E", width, lprecision, val);
 	    } else {
 		engexp = (double) (engind-6)*3;
 		engmant = val/pow(10.0e0,engexp);
-		(void) sprintf(buf,"%*.*fe%s", width-4,
+		snprintf(buf, buflen, "%*.*fe%s", width-4,
 			lprecision, engmant, engmult[engind]);
 	    }
 	}
