@@ -5,16 +5,18 @@
  * $Revision: 7.16 $
  */
 
+#include <stdlib.h>
 #ifdef QREF
 #include <stdio.h>
-char	*header = " Quick Reference";
-char	*revision = "$Revision: 7.16 $";
+static char	*header = " Quick Reference";
+static char	*revision = "$Revision: 7.16 $";
 #else
-#include <curses.h>
+#include <limits.h>
+#include "compat.h"
 #include "sc.h"
 #endif /* QREF */
 
-char *intro[] = {
+static char *intro[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -45,7 +47,7 @@ char *intro[] = {
 (char *)0
 };
 
-char *toggleoptions[] = {
+static char *toggleoptions[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -77,7 +79,7 @@ char *toggleoptions[] = {
 (char *)0
 };
 
-char *setoptions[] = {
+static char *setoptions[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -103,7 +105,7 @@ char *setoptions[] = {
 (char *)0
 };
 
-char *cursor[] = {
+static char *cursor[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -136,7 +138,7 @@ char *cursor[] = {
 };
 
 
-char *cell[] = {
+static char *cell[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -171,7 +173,7 @@ char *cell[] = {
 };
 
 
-char *vi[] = {
+static char *vi[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -207,7 +209,7 @@ char *vi[] = {
 (char *)0
 };
 
-char *file[] = {
+static char *file[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -240,7 +242,7 @@ char *file[] = {
 };
 
 
-char *row[] = {
+static char *row[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -274,7 +276,7 @@ char *row[] = {
 };
 
 
-char *range[] = {
+static char *range[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -308,7 +310,7 @@ char *range[] = {
 };
 
 
-char *misc[] = {
+static char *misc[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -335,7 +337,7 @@ char *misc[] = {
 (char *)0
 };
 
-char *var[] = {
+static char *var[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -367,7 +369,7 @@ char *var[] = {
 (char *)0
 };
 
-char *rangef[] = {
+static char *rangef[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -399,7 +401,7 @@ char *rangef[] = {
 (char *)0
 };
 
-char *numericf[] = {
+static char *numericf[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -432,7 +434,7 @@ char *numericf[] = {
 (char *)0
 };
 
-char *stringf[] = {
+static char *stringf[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -466,7 +468,7 @@ char *stringf[] = {
 };
 
 
-char *finf[] = {
+static char *finf[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -499,7 +501,7 @@ char *finf[] = {
 };
 
 
-char *timef[] = {
+static char *timef[] = {
 " ",
 #if defined(QREF) && defined(TROFF)
 ".SH",
@@ -537,8 +539,7 @@ char *timef[] = {
 static	int	pscreen(char *screen[]);
 
 void
-help()
-{
+help(void) {
     int option;
     char **ns = intro;
 
@@ -589,36 +590,34 @@ pscreen(char *screen[])
     return(nmgetch());
 }
 #else
-char	** pages[] = { intro, toggleoptions, setoptions, cursor, cell, vi,
+static char	** pages[] = { intro, toggleoptions, setoptions, cursor, cell, vi,
 			file, row, range, misc, var, rangef, numericf, stringf,
 			finf, timef, NULL};
 
 int
-main()
-{   int	lineno;
+main() {
+    int	lineno;
     char	***pagep = pages;
 #ifdef TROFF
     int	pageno = 0;
-#endif
 
-#ifdef TROFF
-puts(".nr PS 12");
-puts(".nr VS 14");
-puts(".nr HM 1i");
-puts(".nr FM 1i");
-puts(".nr PO 0.5i");
-printf(".EH '%s%s''%s'\n", SCNAME, header, revision);
-printf(".OH '%s%s''%s'\n", SCNAME, header, revision);
-puts(".EF ''%''");
-puts(".OF ''%''");
-puts(".de Lp");
-puts(".LP");
-puts(".ft CW");
-puts(".na");
-puts(".nf");
-puts("..");
-puts(".P1");
-puts(".LP");
+    puts(".nr PS 12");
+    puts(".nr VS 14");
+    puts(".nr HM 1i");
+    puts(".nr FM 1i");
+    puts(".nr PO 0.5i");
+    printf(".EH '%s%s''%s'\n", SCNAME, header, revision);
+    printf(".OH '%s%s''%s'\n", SCNAME, header, revision);
+    puts(".EF ''%''");
+    puts(".OF ''%''");
+    puts(".de Lp");
+    puts(".LP");
+    puts(".ft CW");
+    puts(".na");
+    puts(".nf");
+    puts("..");
+    puts(".P1");
+    puts(".LP");
 #endif
 
     while (*pagep)
@@ -642,6 +641,6 @@ puts(".LP");
 	if (!(pageno%2)) puts(".bp");
 #endif
     }
-    (void) exit(0);
+    return 0;
 }
 #endif /* QREF */
