@@ -13,35 +13,31 @@
 
 
 #include <sys/types.h>
-#ifdef BSD42
-#include <strings.h>
-#else
-#ifndef SYSIII
+#include <sys/stat.h>
 #include <string.h>
-#endif
-#endif
+#include <math.h>
 
 #if defined(BSD42) || defined(BSD43)
 #include <sys/ioctl.h>
 #endif 
 
-#ifdef IEEE_MATH
-#include <ieeefp.h>
-#endif /* IEEE_MATH */
+#ifdef USE_IEEEFP_H
+# include <ieeefp.h>
+#endif
 
 #include <stdlib.h>
-#include <curses.h>
 #include <signal.h>
 #include <setjmp.h>
 #include <ctype.h>
+#include <unistd.h>
+#include <limits.h>
+#include "compat.h"
 #include "sc.h"
 
-#ifdef NONOTIMEOUT
-#define	notimeout(a1, a2)
-#endif
+static void fpe_trap(int);
 
 #ifdef VMS
-#include "gram_tab.h"
+# include "gram_tab.h"
 typedef union {
     int ival;
     double fval;
@@ -53,11 +49,11 @@ typedef union {
 extern YYSTYPE yylval;
 extern int VMS_read_raw;   /*sigh*/
 #else	/* VMS */
-#if defined(MSDOS)
-#include "y_tab.h"
-#else
-#include "y.tab.h"
-#endif /* MSDOS */
+# if defined(MSDOS)
+#  include "y_tab.h"
+# else
+#  include "y.tab.h"
+# endif /* MSDOS */
 #endif /* VMS */
 
 #ifdef hpux
@@ -69,21 +65,20 @@ jmp_buf fpe_buf;
 
 bool decimal = FALSE;
 
-#ifdef SIGVOID
-void
-#endif
+static void
 fpe_trap(int signo)
 {
+    (void)signo;
 #if defined(i386) && !defined(M_XENIX)
     asm("	fnclex");
     asm("	fwait");
 #else
-#ifdef IEEE_MATH
+# ifdef IEEE_MATH
     (void)fpsetsticky((fp_except)0);	/* Clear exception */
-#endif /* IEEE_MATH */
-#ifdef PC
+# endif /* IEEE_MATH */
+# ifdef PC
     _fpreset();
-#endif
+# endif
 #endif
     longjmp(fpe_buf, 1);
 }
@@ -104,22 +99,22 @@ struct key statres[] = {
 };
 
 int
-yylex()
+yylex(void)
 {
     char *p = line + linelim;
-    int ret;
+    int ret = -1;
     static int isfunc = 0;
     static bool isgoto = 0;
     static bool colstate = 0;
     static int dateflag;
     static char *tokenst = NULL;
-    static int tokenl;
+    static size_t tokenl;
 
-    while (isspace(*p)) p++;
+    while (isspace((int)*p)) p++;
     if (*p == '\0') {
 	isfunc = isgoto = 0;
 	ret = -1;
-    } else if (isalpha(*p) || (*p == '_')) {
+    } else if (isalpha((int)*p) || (*p == '_')) {
 	register char *la;	/* lookahead pointer */
 	register struct key *tblp;
 
@@ -132,23 +127,23 @@ yylex()
 	 *  tokens made up of alphanumeric chars and '_' (a function or
 	 *  token or command or a range name)
 	 */
-	while (isalpha(*p) && isascii(*p)) {
+	while (isalpha((int)*p) && isascii((int)*p)) {
 	    p++;
 	    tokenl++;
 	}
 	la = p;
-	while (isdigit(*la) || (*la == '$'))
+	while (isdigit((int)*la) || (*la == '$'))
 	    la++;
 	/*
 	 * A COL is 1 or 2 char alpha with nothing but digits following
 	 * (no alpha or '_')
 	 */
-	if (!isdigit(*tokenst) && tokenl && tokenl <= 2 && (colstate ||
-		(isdigit(*(la-1)) && !(isalpha(*la) || (*la == '_'))))) {
+	if (!isdigit((int)*tokenst) && tokenl && tokenl <= 2 && (colstate ||
+		(isdigit((int)*(la-1)) && !(isalpha((int)*la) || (*la == '_'))))) {
 	    ret = COL;
 	    yylval.ival = atocol(tokenst, tokenl);
 	} else {
-	    while (isalpha(*p) || (*p == '_') || isdigit(*p)) {
+	    while (isalpha((int)*p) || (*p == '_') || isdigit((int)*p)) {
 		p++;
 		tokenl++;
 	    }
@@ -156,10 +151,13 @@ yylex()
 	    if (!linelim || isfunc) {
 		if (isfunc) isfunc--;
 		for (tblp = linelim ? experres : statres; tblp->key; tblp++)
-		    if (((tblp->key[0]^tokenst[0])&0137)==0
-			    && tblp->key[tokenl]==0) {
-			int i = 1;
-			while (i<tokenl && ((tokenst[i]^tblp->key[i])&0137)==0)
+		    if (((tblp->key[0] ^ tokenst[0]) & 0x5F) == 0) {
+		    /* Commenting the following line makes the search slower */
+		    /* but avoids access outside valid memory. A BST would   */
+		    /* be the better alternative. */
+		    /*  && tblp->key[tokenl] == 0) { */
+			unsigned int i = 1;
+			while (i < tokenl && ((tokenst[i] ^ tblp->key[i]) & 0x5F) == 0)
 			    i++;
 			if (i >= tokenl) {
 			    ret = tblp->val;
@@ -176,17 +174,16 @@ yylex()
 	    if (ret == WORD) {
 		struct range *r;
 		char *path;
-		if (!find_range(tokenst, tokenl,
-			(struct ent *)0, (struct ent *)0, &r)) {
+		if (!find_range(tokenst, tokenl, NULL, NULL, &r)) {
 		    yylval.rval.left = r->r_left;
 		    yylval.rval.right = r->r_right;
 		    if (r->r_is_range)
 		        ret = RANGE;
 		    else
 			ret = VAR;
-		} else if ((path = scxmalloc((unsigned)PATHLEN)) &&
+		} else if ((path = scxmalloc(PATHLEN)) &&
 			plugin_exists(tokenst, tokenl, path)) {
-		    strcat(path, p);
+		    strlcat(path, p, PATHLEN);
 		    yylval.sval = path;
 		    ret = PLUGIN;
 		} else {
@@ -196,7 +193,7 @@ yylex()
 		}
 	    }
 	}
-    } else if ((*p == '.') || isdigit(*p)) {
+    } else if ((*p == '.') || isdigit((int)*p)) {
 #ifdef SIGVOID
 	void (*sig_save)();
 #else
@@ -226,7 +223,7 @@ yylex()
 		do {
 		    v = v*10.0 + (double) ((unsigned) *p - '0');
 		    tokenl++;
-		} while (isdigit(*++p));
+		} while (isdigit((int)*++p));
 		if (dateflag) {
 		    ret = NUMBER;
 		    yylval.ival = (int)v;
@@ -236,19 +233,19 @@ yylex()
 		 *  .'s as tokens instead of interpreting them as decimal
 		 *  points.  dateflag counts the .'s as they're returned.
 		 */
-		} else if (*p=='.' && isdigit(*(p+1)) && (*(p+2)=='.' ||
-			(isdigit(*(p+2)) && *(p+3)=='.'))) {
+		} else if (*p=='.' && isdigit((int)*(p+1)) && (*(p+2)=='.' ||
+			(isdigit((int)*(p+2)) && *(p+3)=='.'))) {
 		    ret = NUMBER;
 		    yylval.ival = (int)v;
 		    dateflag = 2;
 		} else if (*p == 'e' || *p == 'E') {
-		    while (isdigit(*++p)) /* */;
-		    if (isalpha(*p) || *p == '_') {
+		    while (isdigit((int)*++p)) /* */;
+		    if (isalpha((int)*p) || *p == '_') {
 			linelim = p - line;
 			return (yylex());
 		    } else
 			ret = FNUMBER;
-		} else if (isalpha(*p) || *p == '_') {
+		} else if (isalpha((int)*p) || *p == '_') {
 		    linelim = p - line;
 		    return (yylex());
 		}
@@ -256,7 +253,13 @@ yylex()
 	    if ((!dateflag && *p=='.') || ret == FNUMBER) {
 		ret = FNUMBER;
 		yylval.fval = strtod(nstart, &p);
-		if (!finite(yylval.fval))
+		if (!
+#ifdef HAVE_ISFINITE
+		  isfinite(
+#else
+		  finite(
+#endif
+		  yylval.fval))
 		    ret = K_ERR;
 		else
 		    decimal = TRUE;
@@ -284,7 +287,7 @@ yylex()
         ptr = p+1;	/* "string" or "string\"quoted\"" */
         while (*ptr && ((*ptr != '"') || (*(ptr-1) == '\\')))
 	    ptr++;
-        ptr = scxmalloc((unsigned)(ptr-p));
+        ptr = scxmalloc(ptr-p);
 	yylval.sval = ptr;
 	p++;
 	while (*p && ((*p != '"') ||
@@ -316,28 +319,26 @@ yylex()
 */
 
 int
-plugin_exists(char *name, int len, char *path)
+plugin_exists(char *name, size_t len, char *path)
 {
 #ifndef MSDOS
-    FILE *fp;
-    static char *HomeDir;
+    struct stat sb;
+    static char *homedir;
 
-    if ((HomeDir = getenv("HOME"))) {
-	strcpy((char *)path, HomeDir);
-	strcat((char *)path, "/.sc/plugins/");
-	strncat((char *)path, name, len);
-	if (fp = fopen((char *)path, "r")) {
-	    fclose(fp);
+    if ((homedir = getenv("HOME"))) {
+	if (strlcpy(path, homedir, len) >= len
+          || strlcat(path, "/.sc/plugins/", len) >= len
+          || strlcat(path, name, len) >= len)
+	    return 0;
+        if (!stat(path, &sb))
 	    return 1;
-	}
     }
-    strcpy((char *)path, LIBDIR);
-    strcat((char *)path, "/plugins/");
-    strncat((char *)path, name, len);
-    if (fp = fopen((char *)path, "r")) {
-	fclose(fp);
+    if (strlcpy(path, LIBDIR, len) >= len
+      || strlcat(path, "/plugins/", len) >= len
+      || strlcat(path, name, len) >= len)
+	return 0;
+    if (!stat(path, &sb))
 	return 1;
-    }
 #endif
     return 0;
 }
@@ -356,10 +357,10 @@ atocol(char *string, int len)
 {
     register int col;
 
-    col = (toupper(string[0])) - 'A';
+    col = (toupper((int)string[0])) - 'A';
 
     if (len == 2)		/* has second char */
-	col = ((col + 1) * 26) + ((toupper(string[1])) - 'A');
+	col = ((col + 1) * 26) + ((toupper((int)string[1])) - 'A');
 
     return (col);
 }
@@ -368,29 +369,29 @@ atocol(char *string, int len)
 #ifdef SIMPLE
 
 void
-initkbd()
+initkbd(void)
 {}
 
 void
-kbd_again()
+kbd_again(void)
 {}
 
 void
-resetkbd()
+resetkbd(void)
 {}
 
-#ifndef VMS
+# ifndef VMS
 
 int
-nmgetch()
+nmgetch(void)
 {
     return (getchar());
 }
 
-#else /* VMS */
+# else /* VMS */
 
 int
-nmgetch()
+nmgetch(void)
 /*
    This is not perfect, it doesn't move the cursor when goraw changes
    over to deraw, but it works well enough since the whole sc package
@@ -406,7 +407,7 @@ nmgetch()
     short c;
     static int key_id=0;
     int status;
-#define VMScheck(a) {if (~(status = (a)) & 1) VMS_MSG (status);}
+#  define VMScheck(a) {if (~(status = (a)) & 1) VMS_MSG (status);}
 
     if (VMS_read_raw) {
       VMScheck(smg$read_keystroke (&stdkb->_id, &c, 0, 0, 0));
@@ -429,11 +430,11 @@ VMS_MSG (int status)
    Routine to put out the VMS operating system error (if one occurs).
 */
 {
-#include <descrip.h>
+#  include <descrip.h>
    char errstr[81], buf[120];
    $DESCRIPTOR(errdesc, errstr);
    short length;
-#define err_out(msg) fprintf (stderr,msg)
+#  define err_out(msg) fprintf (stderr,msg)
 
 /* Check for no error or standard error */
 
@@ -441,19 +442,20 @@ VMS_MSG (int status)
 	status = status & 0x8000 ? status & 0xFFFFFFF : status & 0xFFFF;
 	if (SYS$GETMSG(status, &length, &errdesc, 1, 0) == SS$_NORMAL) {
 	    errstr[length] = '\0';
-	    (void) sprintf(buf, "<0x%x> %s", status, errdesc.dsc$a_pointer);
+	    snprintf(buf, sizeof buf, "<0x%x> %s", status,
+	        errdesc.dsc$a_pointer);
 	    err_out(buf);
 	} else
 	    err_out("System error");
     }
 }
-#endif /* VMS */
+# endif /* VMS */
 
 #else /*SIMPLE*/
 
-#if defined(BSD42) || defined (SYSIII) || defined(BSD43)
+# if defined(BSD42) || defined (SYSIII) || defined(BSD43)
 
-#define N_KEY 4
+#  define N_KEY 4
 
 struct key_map {
     char *k_str;
@@ -465,16 +467,14 @@ struct key_map km[N_KEY];
 
 char keyarea[N_KEY*30];
 
-char *tgetstr();
-char *getenv();
 char *ks;
 char ks_buf[20];
 char *ke;
 char ke_buf[20];
 
-#ifdef TIOCSLTC
+#  ifdef TIOCSLTC
 struct ltchars old_chars, new_chars;
-#endif
+#  endif
 
 char dont_use[] = {
     ctl('['), ctl('a'), ctl('b'), ctl('c'), ctl('e'), ctl('f'), ctl('g'),
@@ -483,14 +483,13 @@ char dont_use[] = {
     ctl('x'), ctl('z'), 0
 };
 
-void
-charout(int c)
-{
-    (void)putchar(c);
+int
+charout(int c) {
+    return putchar(c);
 }
 
 void
-initkbd()
+initkbd(void)
 {
     register struct key_map *kp;
     register i,j;
@@ -512,13 +511,13 @@ initkbd()
 
     ktmp = tgetstr("ks",&p);
     if (ktmp)  {
-	(void) strcpy(ks_buf, ktmp);
+	strlcpy(ks_buf, ktmp, sizeof ks_buf);
 	ks = ks_buf;
 	tputs(ks, 1, charout);
     }
     ktmp = tgetstr("ke",&p);
     if (ktmp)  {
-	(void) strcpy(ke_buf, ktmp);
+	strlcpy(ke_buf, ktmp, sizeof ke_buf);
 	ke = ke_buf;
     }
 
@@ -536,7 +535,7 @@ initkbd()
     }
 
 
-#ifdef TIOCSLTC
+#  ifdef TIOCSLTC
     (void)ioctl(fileno(stdin), TIOCGLTC, (char *)&old_chars);
     new_chars = old_chars;
     if (old_chars.t_lnextc == ctl('v'))
@@ -544,34 +543,33 @@ initkbd()
     if (old_chars.t_rprntc == ctl('r'))
 	new_chars.t_rprntc = -1;
     (void)ioctl(fileno(stdin), TIOCSLTC, (char *)&new_chars);
-#endif
+#  endif
 }
 
 void
-kbd_again()
+kbd_again(void)
 {
     if (ks) 
 	tputs(ks, 1, charout);
 
-#ifdef TIOCSLTC
+#  ifdef TIOCSLTC
     (void)ioctl(fileno(stdin), TIOCSLTC, (char *)&new_chars);
-#endif
+#  endif
 }
 
 void
-resetkbd()
+resetkbd(void)
 {
     if (ke) 
 	tputs(ke, 1, charout);
 
-#ifdef TIOCSLTC
+#  ifdef TIOCSLTC
     (void)ioctl(fileno(stdin), TIOCSLTC, (char *)&old_chars);
-#endif
+#  endif
 }
 
 int
-nmgetch() 
-{
+nmgetch(void) {
     register int c;
     register struct key_map *kp;
     register struct key_map *biggest;
@@ -582,11 +580,11 @@ nmgetch()
     static char dumpbuf[10];
     static char *dumpindex;
 
-#ifdef SIGVOID
-    void time_out();
-#else
-    int time_out();
-#endif
+#  ifdef SIGVOID
+    void time_out(int);
+#  else
+    int time_out(int);
+#  endif
 
     if (dumpindex && *dumpindex)
 	return (*dumpindex++);
@@ -640,42 +638,45 @@ nmgetch()
     return(c);
 }
 
+# endif /* if defined(BSD42) || defined (SYSIII) || defined(BSD43) */
+
+void
+initkbd(void)
+{
+    keypad(stdscr, TRUE);
+#ifndef NONOTIMEOUT
+    notimeout(stdscr,TRUE);
 #endif
-
-#if defined(SYSV2) || defined(SYSV3) || defined(MSDOS)
-
-void
-initkbd()
-{
-    keypad(stdscr, TRUE);
-    notimeout(stdscr,TRUE);
 }
 
 void
-kbd_again()
+kbd_again(void)
 {
     keypad(stdscr, TRUE);
+#ifndef NONOTIMEOUT
     notimeout(stdscr,TRUE);
+#endif
 }
 
 void
-resetkbd()
+resetkbd(void)
 {
     keypad(stdscr, FALSE);
+#ifndef NONOTIMEOUT
     notimeout(stdscr, FALSE);
+#endif
 }
 
 int
-nmgetch()
-{
+nmgetch(void) {
     register int c;
 
     c = getch();
     switch (c) {
-#ifdef KEY_SELECT
+# ifdef KEY_SELECT
 	case KEY_SELECT:	c = 'm';	break;
-#endif
-#ifdef KEY_C1
+# endif
+# ifdef KEY_C1
 /* This stuff works for a wyse wy75 in ANSI mode under 5.3.  Good luck. */
 /* It is supposed to map the curses keypad back to the numeric equiv. */
 
@@ -700,14 +701,12 @@ nmgetch()
  *
  *
  */
-#endif
+# endif
 	default:	break;
     }
     return (c);
 }
 
-#endif /* SYSV2 || SYSV3 */
-
 #endif /* SIMPLE */
 
 #ifdef SIGVOID
@@ -717,5 +716,6 @@ int
 #endif
 time_out(int signo)
 {
+    (void)signo;
     longjmp(wakeup, 1);
 }
