@@ -7,23 +7,12 @@
  *		$Revision: 7.16 $
  */
 
-#include <curses.h>
 #include <unistd.h>
-#include "sc.h"
-
-#ifdef BSD42
-#include <strings.h>
-#include <sys/time.h>
-#ifndef strchr
-#define strchr index
-#endif
-#else
 #include <time.h>
-#ifndef SYSIII
 #include <string.h>
-#endif
-#endif
-
+#include <limits.h>
+#include "compat.h"
+#include "sc.h"
 
 void
 getnum(int r0, int c0, int rn, int cn, int fd)
@@ -36,16 +25,17 @@ getnum(int r0, int c0, int rn, int cn, int fd)
 	for (c = c0, pp = ATBL(tbl, r, c); c <= cn; pp++, c++) {
 	    *line = '\0';
 	    p = *pp;
-	    if (p)
+	    if (p) {
 		if (p->cellerror)
-		    sprintf(line, "%s", (*pp)->cellerror == CELLERROR ?
+		    snprintf(line, sizeof line, "%s", (*pp)->cellerror == CELLERROR ?
 			    "ERROR" : "INVALID");
-		else if (p->flags & is_valid)
-		    sprintf(line, "%.15g", p->v);
+		else if (p->flags & IS_VALID)
+		    snprintf(line, sizeof line, "%.15g", p->v);
+	    }
 	    if (c < cn)
-		strcat(line, "\t");
+		strlcat(line, "\t", sizeof line);
 	    else
-		strcat(line, "\n");
+		strlcat(line, "\n", sizeof line);
 	    write(fd, line, strlen(line));
 	    if (brokenpipe) {
 		linelim = -1;
@@ -69,9 +59,9 @@ fgetnum(int r0, int c0, int rn, int cn, int fd)
 	    p = *pp;
 	    if (p) {
 		if (p->cellerror)
-		    sprintf(line, "%s", p->cellerror == CELLERROR ?
+		    snprintf(line, sizeof line, "%s", p->cellerror == CELLERROR ?
 			    "ERROR" : "INVALID");
-		else if (p->flags & is_valid) {
+		else if (p->flags & IS_VALID) {
 		    if (p->format) {
 			if (*(p->format) == ctl('d')) {
 			    time_t i = (time_t) (p->v);
@@ -86,9 +76,9 @@ fgetnum(int r0, int c0, int rn, int cn, int fd)
 		}
 	    }
 	    if (c < cn)
-		strcat(line, "\t");
+		strlcat(line, "\t", sizeof line);
 	    else
-		strcat(line, "\n");
+		strlcat(line, "\n", sizeof line);
 	    write(fd, line, strlen(line));
 	    if (brokenpipe) {
 		linelim = -1;
@@ -109,11 +99,11 @@ getstring(int r0, int c0, int rn, int cn, int fd)
 	for (c = c0, pp = ATBL(tbl, r, c); c <= cn; pp++, c++) {
 	    *line = '\0';
 	    if (*pp && (*pp)->label)
-		sprintf(line, "%s", (*pp)->label);
+		snprintf(line, sizeof line, "%s", (*pp)->label);
 	    if (c < cn)
-		strcat(line, "\t");
+		strlcat(line, "\t", sizeof line);
 	    else
-		strcat(line, "\n");
+		strlcat(line, "\n", sizeof line);
 	    write(fd, line, strlen(line));
 	    if (brokenpipe) {
 		linelim = -1;
@@ -143,9 +133,9 @@ getexp(int r0, int c0, int rn, int cn, int fd)
 		    *line = '\0';
 	    }
 	    if (c < cn)
-		strcat(line, "\t");
+		strlcat(line, "\t", sizeof line);
 	    else
-		strcat(line, "\n");
+		strlcat(line, "\n", sizeof line);
 	    write(fd, line, strlen(line));
 	    if (brokenpipe) {
 		linelim = -1;
@@ -159,7 +149,7 @@ getexp(int r0, int c0, int rn, int cn, int fd)
 void
 getformat(int col, int fd)
 {
-    sprintf(line, "%d %d %d\n", fwidth[col], precision[col], realfmt[col]);
+    snprintf(line, sizeof line, "%d %d %d\n", fwidth[col], precision[col], realfmt[col]);
     write(fd, line, strlen(line));
     linelim = -1;
 }
@@ -174,11 +164,11 @@ getfmt(int r0, int c0, int rn, int cn, int fd)
 	for (c = c0, pp = ATBL(tbl, r, c); c <= cn; pp++, c++) {
 	    *line = '\0';
 	    if (*pp && (*pp)->format)
-		sprintf(line, "%s", (*pp)->format);
+		snprintf(line, sizeof line, "%s", (*pp)->format);
 	    if (c < cn)
-		strcat(line, "\t");
+		strlcat(line, "\t", sizeof line);
 	    else
-		strcat(line, "\n");
+		strlcat(line, "\n", sizeof line);
 	    write(fd, line, strlen(line));
 	    if (brokenpipe) {
 		linelim = -1;
@@ -193,16 +183,18 @@ void
 getframe(int fd)
 {
     struct frange *fr;
+    size_t l;
 
     *line = '\0';
     if ((fr = find_frange(currow, curcol))) {
-	sprintf(line, "%s", r_name(fr->or_left->row, fr->or_left->col,
+	snprintf(line, sizeof line, "%s", r_name(fr->or_left->row, fr->or_left->col,
 		fr->or_right->row, fr->or_right->col));
-	strcat(line, " ");
-	sprintf(line + strlen(line), "%s", r_name(fr->ir_left->row,
+	strlcat(line, " ", sizeof line);
+	l = strlen(line);
+	snprintf(line + l, sizeof(line) - l, "%s", r_name(fr->ir_left->row,
 		fr->ir_left->col, fr->ir_right->row, fr->ir_right->col));
     }
-    strcat(line, "\n");
+    strlcat(line, "\n", sizeof line);
     write(fd, line, strlen(line));
     linelim = -1;
 }
@@ -215,7 +207,7 @@ getrange(char *name, int fd)
 
     *line = '\0';
     if (!find_range(name, strlen(name), (struct ent *)0, (struct ent *)0, &r)) {
-	sprintf(line, "%s%s%s%d",
+	snprintf(line, sizeof line, "%s%s%s%d",
 		r->r_left.vf & FIX_COL ? "$" : "",
 		coltoa(r->r_left.vp->col),
 		r->r_left.vf & FIX_ROW ? "$" : "",
@@ -224,7 +216,7 @@ getrange(char *name, int fd)
 	    p = line;
 	    while (*p)
 		p++;
-	    sprintf(p, ":%s%s%s%d",
+	    snprintf(p, sizeof(line) - (p - line), ":%s%s%s%d",
 		    r->r_right.vf & FIX_COL ? "$" : "",
 		    coltoa(r->r_right.vp->col),
 		    r->r_right.vf & FIX_ROW ? "$" : "",
@@ -245,7 +237,7 @@ getrange(char *name, int fd)
 	/*                                              */
 	/************************************************/
     }
-    strcat(line, "\n");
+    strlcat(line, "\n", sizeof line);
     write(fd, line, strlen(line));
     linelim = -1;
 }
@@ -266,8 +258,8 @@ doeval(struct enode *e, char *fmt, int row, int col, int fd)
 	} else
 	    format(fmt, precision[col], v, line, FBUFLEN);
     } else
-	sprintf(line, "%.15g", v);
-    strcat(line, "\n");
+	snprintf(line, sizeof line, "%.15g", v);
+    strlcat(line, "\n", sizeof line);
     write(fd, line, strlen(line));
     linelim = -1;
 
@@ -308,15 +300,14 @@ doquery(char *s, char *data, int fd)
 
     line[0] = '\0';
     linelim = -1;
-    error("");
+    CLEAR_LINE;
     update(0);
 
     if (s) scxfree(s);
 }
 
 void
-dogetkey()
-{
+dogetkey(void) {
     int c, len;
 
     goraw();
@@ -324,12 +315,13 @@ dogetkey()
     deraw(0);
 
     if (c < 256) {
-	sprintf(line, "%c", c);
+	snprintf(line, sizeof line, "%c", c);
 	len = 1;
+#ifdef HAVE_CURSES_KEYNAME
     } else if (c >= KEY_MIN && c <= KEY_MAX) {
 	int i, j;
 	line[0] = '\0';
-	sprintf(line + 1, "%s\n", keyname(c));
+	snprintf(line + 1, sizeof(line) - 1, "%s\n", keyname(c));
 	for (i = 1, j = 5; line[j-1]; ) {
 	    if (line[j] == '(' || line[j] == ')')
 		j++;
@@ -337,9 +329,10 @@ dogetkey()
 		line[i++] = line[j++];
 	}
 	len = strlen(line + 1) + 1;
+#endif
     } else {
 	line[0] = '0';
-	sprintf(line + 1, "UNKNOWN KEY");
+	snprintf(line + 1, sizeof(line) - 1, "UNKNOWN KEY");
 	len = strlen(line + 1) + 1;
     }
 
@@ -351,10 +344,10 @@ void
 dostat(int fd)
 {
     *line = '\0';
-    if (modflg)			sprintf(line, "m");
-    if (isatty(STDIN_FILENO))	strcat(line, "i");
-    if (isatty(STDOUT_FILENO))	strcat(line, "o");
-    strcat(line, "\n");
+    if (modflg)			snprintf(line, sizeof line, "m");
+    if (isatty(STDIN_FILENO))	strlcat(line, "i", sizeof line);
+    if (isatty(STDOUT_FILENO))	strlcat(line, "o", sizeof line);
+    strlcat(line, "\n", sizeof line);
     write(fd, line, strlen(line));
     linelim = -1;
 }
