@@ -17,12 +17,25 @@
  *  Author: Robert Bond
  *  Adjustments: Jeff Buhrt, Eric Putz and Chuck Martin
  */
-char *rev = "$Revision: 7.16 $";
+
+#include "version.c"
 
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+# if !HAVE__BOOL
+#  define _Bool int
+# endif
+# define bool _Bool
+# define true 1
+# define false 0
+#endif
+#include <limits.h>
+#include <string.h>
 #include "sc.h"
 
 #define END	0
@@ -35,35 +48,37 @@ char	*coltoa(int col);
 char	*progname;
 int	getrow(char *p);
 int	getcol(char *p);
-int	scan();
+static int scan(void);
 
 int *fwidth;
 int *precision;
 int maxcols;
 int *realfmt;
 
-int curlen;
-int curcol, coff;
-int currow, roff;
-int first;
-int effr, effc;
+static int curlen;
+int curcol;
+static int coff;
+int currow;
+static int roff;
+static int first;
+static int effr, effc;
 
 /* option flags reset */
-int colfirst = FALSE;
-int leftadj = FALSE;
-int r0 = 0;
-int c0 = 0;
-int rinc = 1;
-int cinc = 1;
-int len = 20000;
-char delim1 = ' ';
-char delim2 = '\t';
-int strip_delim = TRUE;
-int drop_format = FALSE;
-int strnums	= FALSE;
-int plainnums	= FALSE;
+static int colfirst = FALSE;
+static int leftadj = FALSE;
+static int r0 = 0;
+static int c0 = 0;
+static int rinc = 1;
+static int cinc = 1;
+static int len = 20000;
+static char delim1 = ' ';
+static char delim2 = '\t';
+static int strip_delim = TRUE;
+static int drop_format = FALSE;
+static int strnums	= FALSE;
+static int plainnums	= FALSE;
 
-char token[1000];
+static char token[1000];
 
 int
 main(int argc, char **argv)
@@ -111,15 +126,16 @@ main(int argc, char **argv)
 	    plainnums = TRUE;
 	    break;
 	case 'v':
-	    (void) fprintf(stderr,"%s: %s\n", progname, rev);
+	    fprintf(stderr,"%s: %s\n", progname, rev);
+	    return 0;
 	default:
-	    (void) fprintf(stderr,"Usage: %s [-rkfLSPv] [-s v] [-R i] [-C i] [-n i] [-d c]\n", progname);
 	    exit(1);
         }
     }
 
     if (optind < argc) {
-	    (void) fprintf(stderr,"Usage: %s [-rL] [-s v] [-R i] [-C i] [-n i] [-d c]\n", progname);
+	    fprintf(stderr, "%s: %d more argument(s) than expected\n",
+		progname, argc - optind);
 	    exit(1);
     }
 
@@ -237,9 +253,8 @@ main(int argc, char **argv)
     }
 }
 
-int
-scan()
-{
+static int
+scan(void) {
     register int c;
     register char *p;
     register int founddigit;
@@ -316,13 +331,13 @@ getcol(char *p)
     col = 0;
     if (!p)
 	return (0);
-    while (*p && !isalpha(*p)) 
+    while (*p && !isalpha((int)*p)) 
 	p++; 
     if (!*p)
 	return (0);
-    col = (toupper(*p) - 'A');
-    if (isalpha(*++p)) 
-	col = (col + 1)*26 + (toupper(*p) - 'A');
+    col = (toupper((int)*p) - 'A');
+    if (isalpha((int)*++p)) 
+	col = (col + 1)*26 + (toupper((int)*p) - 'A');
     return (col);
 }
 
@@ -335,12 +350,12 @@ getrow(char *p)
     row = 0;
     if (!p)
 	return (0);
-    while (*p && !isdigit(*p))
+    while (*p && !isdigit((int)*p))
 	p++; 
     if (!*p)
 	return (0);
-    while (*p && isdigit(*p))
-    {	row = row * 10 + *p - '0';
+    while (*p && isdigit((int)*p)) {
+	row = row * 10 + *p - '0';
 	p++;
     }
     return (row);
