@@ -8,16 +8,12 @@
  */
 
 #include <sys/types.h>
-#ifdef BSD42
-#include <strings.h>
-#else
-#ifndef SYSIII
 #include <string.h>
-#endif
-#endif
-
 #include <stdio.h>
 #include <ctype.h>
+#include <unistd.h>
+#include <limits.h>
+#include "compat.h"
 #include "sc.h"
 
 static	struct range *rng_base;
@@ -64,24 +60,24 @@ add_range(char *name, struct ent_ptr left, struct ent_ptr right, int is_range)
     }
 
     for (p = name; *p; p++)
-	if (!(isalpha(*p) || isdigit(*p) || *p == '_')) {
+	if (!(isalpha((int)*p) || isdigit((int)*p) || *p == '_')) {
 	    error("Invalid range name \"%s\" - illegal combination", name);
 	    scxfree(name);
 	    return;
 	}
  
     p = name;
-    if (isdigit(*p) || (isalpha(*p++) && (isdigit(*p) ||
-		(isalpha(*p++) && isdigit(*p))))) {
+    if (isdigit((int)*p) || (isalpha((int)*p++) && (isdigit((int)*p) ||
+		(isalpha((int)*p++) && isdigit((int)*p))))) {
 	if (*name == '0' && (name[1] == 'x' || name[1] == 'X')) {
 	    ++p;
-	    while (isxdigit(*++p)) /* */;
+	    while (isxdigit((int)*++p)) /* */;
 	    if (*p == 'p' || *p == 'P')
-		while (isxdigit(*++p)) /* */;
+		while (isxdigit((int)*++p)) /* */;
 	} else {
-	    while (isdigit(*++p)) /* */;
-	    if (isdigit(*name) && (*p == 'e' || *p == 'E'))
-		while (isdigit(*++p)) /* */;
+	    while (isdigit((int)*++p)) /* */;
+	    if (isdigit((int)*name) && (*p == 'e' || *p == 'E'))
+		while (isdigit((int)*++p)) /* */;
 	}
 	if (!(*p)) {
 	    error("Invalid range name \"%s\" - ambiguous", name);
@@ -96,7 +92,7 @@ add_range(char *name, struct ent_ptr left, struct ent_ptr right, int is_range)
 		label(rcp, name, 0);
     }
 
-    r = (struct range *)scxmalloc((unsigned)sizeof(struct range));
+    r = scxmalloc(sizeof(struct range));
     r->r_name = name;
     r->r_left = left;
     r->r_right = right;
@@ -146,8 +142,7 @@ del_range(struct ent *left, struct ent *right)
 }
 
 void
-clean_range()
-{
+clean_range(void) {
     register struct range *r;
     register struct range *nextr;
 
@@ -183,7 +178,7 @@ find_range(char *name, int len, struct ent *lmatch, struct ent *rmatch,
 		return (cmp);
 	    *rng = r;
 	    if (cmp == 0)
-		if (!exact || strlen(r->r_name) == len)
+		if (!exact || strlen(r->r_name) == (size_t)len)
 		    return (cmp);
 	}
 	return (-1);
@@ -199,7 +194,7 @@ find_range(char *name, int len, struct ent *lmatch, struct ent *rmatch,
 }
 
 void
-sync_ranges()
+sync_ranges(void)
 {
     int i, j;
     struct range *r;
@@ -304,7 +299,7 @@ v_name(int row, int col)
     if (!find_range((char *)0, 0, v, v, &r)) {
 	return (r->r_name);
     } else {
-        (void) sprintf(buf, "%s%d", coltoa(col), row);
+        snprintf(buf, sizeof buf, "%s%d", coltoa(col), row);
 	return (buf);
     }
 }
@@ -321,15 +316,16 @@ r_name(int r1, int c1, int r2, int c2)
     if (!find_range((char *)0, 0, v1, v2, &r)) {
 	return (r->r_name);
     } else {
-        (void) sprintf(buf, "%s", v_name(r1, c1));
-	(void) sprintf(buf+strlen(buf), ":%s", v_name(r2, c2));
+	size_t l;
+        snprintf(buf, sizeof buf, "%s", v_name(r1, c1));
+	l = strlen(buf);
+	snprintf(buf + l, sizeof(buf) - l, ":%s", v_name(r2, c2));
 	return (buf);
     }
 }
 
 int
-are_ranges()
-{
+are_ranges(void) {
     return (rng_base != 0);
 }
 
