@@ -14,6 +14,7 @@
 #include <stdio.h>
 #endif
 
+#define CLEAR_LINE error("%s", "") /* suppress warning on NetBSD curses */
 #define	ATBL(tbl, row, col)	(*(tbl + row) + (col))
 
 #define MINROWS 100 	/* minimum size at startup */
@@ -43,10 +44,10 @@
 #ifdef PSC
 # define error(msg)	fprintf(stderr, msg);
 #else
-# define error isatty(fileno(stdout)) && !move(1,0) && !clrtoeol() && printw
+# define error if (isatty(fileno(stdout)) && !move(1,0) && !clrtoeol()) printw
 #endif
 #define	FBUFLEN	1024	/* buffer size for a single field */
-#define	PATHLEN	1024	/* maximum path length */
+#define PATHLEN (PATH_MAX < 8192 ? 8192 : PATH_MAX) /* maximum path length */
 
 #ifndef DFLT_PAGER
 #define	DFLT_PAGER "more"	/* more is probably more widespread than less */
@@ -63,8 +64,11 @@
 #define color_set(c, o)		attron(COLOR_PAIR(c))
 #endif
 
-#if !defined(attr_get) || defined(NCURSES_VERSION) && NCURSES_VERSION_MAJOR < 5
-#undef attr_get
+#if !defined(HAVE_ATTR_T) && defined(_COMPAT_H) /* Not defined for psc */
+typedef chtype attr_t;
+#endif
+
+#if !defined(HAVE_ATTR_GET) && !defined(NO_ATTR_GET)
 #define attr_get(a, p, o)	((void)((a) != 0 && (*(a) = stdscr->_attrs)), \
 				(void)((p) != 0 && \
 				(*(p) = PAIR_NUMBER(stdscr->_attrs))), OK)
@@ -107,18 +111,18 @@ struct range_s {
 
 /*
  * Some not too obvious things about the flags:
- *    is_valid means there is a valid number in v.
- *    is_locked means that the cell cannot be edited.
- *    is_label set means it points to a valid constant string.
- *    is_strexpr set means expr yields a string expression.
- *    If is_strexpr is not set, and expr points to an expression tree, the
+ *    IS_VALID means there is a valid number in v.
+ *    IS_LOCKED means that the cell cannot be edited.
+ *    IS_LABEL set means it points to a valid constant string.
+ *    IS_STREXPR set means expr yields a string expression.
+ *    If IS_STREXPR is not set, and expr points to an expression tree, the
  *        expression yields a numeric expression.
  *    So, either v or label can be set to a constant. 
  *        Either (but not both at the same time) can be set from an expression.
  */
 
 #define VALID_CELL(p, r, c) ((p = *ATBL(tbl, r, c)) && \
-			     ((p->flags & is_valid) || p->label))
+			     ((p->flags & IS_VALID) || p->label))
 
 /* info for each cell, only alloc'd when something is stored in a cell */
 struct ent {
@@ -297,15 +301,15 @@ struct go_save {
 #define WHITE (OP_BASE + 76)
 
 /* flag values */
-#define is_valid     0001
-#define is_changed   0002
-#define is_strexpr   0004
-#define is_leftflush 0010
-#define is_deleted   0020
-#define is_locked    0040
-#define is_label     0100
-#define is_cleared   0200
-#define may_sync     0400
+#define IS_VALID     0001
+#define IS_CHANGED   0002
+#define IS_STREXPR   0004
+#define IS_LEFTFLUSH 0010
+#define IS_DELETED   0020
+#define IS_LOCKED    0040
+#define IS_LABEL     0100
+#define IS_CLEARED   0200
+#define MAY_SYNC     0400
 
 /* cell error (1st generation (ERROR) or 2nd+ (INVALID)) */
 #define	CELLOK		0
@@ -343,7 +347,7 @@ struct go_save {
 #define	GROWBOTH	4	/* grow both */
 extern	struct ent ***tbl;	/* data table ref. in vmtbl.c and ATBL() */
 
-extern	char curfile[];
+extern	char curfile[PATHLEN];
 extern	int arg;
 extern	int strow, stcol;
 extern	int currow, curcol;
@@ -363,7 +367,7 @@ extern	char *colformat[10];
 extern	char *col_hidden;
 extern	char *row_hidden;
 extern	char line[FBUFLEN];
-extern	int linelim;
+extern	ssize_t linelim;
 extern	int changed;
 extern	struct ent *delbuf[DELBUFSIZE];
 extern	char *delbuffmt[DELBUFSIZE];
@@ -378,37 +382,46 @@ extern	int usecurses;
 extern	int brokenpipe;		/* Set to true if SIGPIPE is received */
 extern  char dpoint;	/* country-dependent decimal point from locale */
 extern  char thsep;	/* country-dependent thousands separator from locale */
+extern	char histfile[PATHLEN];
+extern	int lastmx, lastmy;	/* Screen address of the cursor */
+extern	int lastcol, lcols;	/* Spreadsheet Column the cursor was in last */
+extern	int lastendrow;		/* Last bottom row of screen */
+extern	int framerows;		/* Rows in current frame */
+extern	int framecols;		/* Columns in current frame */
+extern	char mode_ind;		/* Mode indicator */
+extern	int	seenerr;
+extern	char	*rev;
 
-extern	FILE *openfile(char *fname, int *rpid, int *rfd);
+extern	FILE *openfile(char *, size_t, int *, int *);
 extern	char *coltoa(int col);
 extern	char *findplugin(char *ext, char type);
-extern	char *findhome(char *path);
+extern	char *findhome(char *, size_t);
 extern	char *r_name(int r1, int c1, int r2, int c2);
-extern	char *scxmalloc(unsigned n);
-extern	char *scxrealloc(char *ptr, unsigned n);
+extern	void *scxmalloc(size_t n);
+extern	void *scxrealloc(void *ptr, size_t n);
 extern	char *seval(register struct enode *se);
 extern	char *v_name(int row, int col);
 extern	double eval(register struct enode *e);
 extern	int any_locked_cells(int r1, int c1, int r2, int c2);
-extern	int are_colors();
-extern	int are_frames();
-extern	int are_ranges();
+extern	int are_colors(void);
+extern	int are_frames(void);
+extern	int are_ranges(void);
 extern	int atocol(char *string, int len);
 extern	int creadfile(char *save, int  eraseflg);
 extern	int cwritefile(char *fname, int r0, int c0, int rn, int cn);
-extern	int engformat(int fmt, int width, int lprecision, double val,
+extern	bool engformat(int fmt, int width, int lprecision, double val,
 	char *buf, int buflen);
 extern	int etype(register struct enode *e);
 extern	int find_range(char *name, int len, struct ent *lmatch,
 	struct ent *rmatch, struct range **rng);
-extern	int format(char *fmt, int lprecision, double val, char *buf,
-	int buflen);
+extern	bool format(char *fmt, int lprecision, double val, char *buf,
+	size_t buflen);
 extern	int get_rcqual(int ch);
 extern	int growtbl(int rowcol, int toprow, int topcol);
 extern	int locked_cell(int r, int c);
 extern	int modcheck(char *endstr);
-extern	int nmgetch();
-extern	int plugin_exists(char *name, int len, char *path);
+extern	int nmgetch(void);
+extern	int plugin_exists(char *name, size_t len, char *path);
 extern	int readfile(char *fname, int eraseflg);
 extern	int writefile(char *fname, int r0, int c0, int rn, int cn);
 extern	int yn_ask(char *msg);
@@ -424,7 +437,7 @@ extern	struct enode *new_var(int op, struct ent_ptr a1);
 extern	struct ent *lookat(int row, int col);
 extern	struct crange *find_crange(int row, int col);
 extern	struct frange *find_frange(int row, int col);
-extern	void EvalAll();
+extern	void EvalAll(void);
 extern	void add_crange(struct ent *r_left, struct ent *r_right, int pair);
 extern	void add_frange(struct ent *or_left, struct ent *or_right,
 	struct ent *ir_left, struct ent *ir_right, int toprows, int bottomrows,
@@ -437,13 +450,13 @@ extern	void backrow(int arg);
 extern	void change_color(int pair, struct enode *e);
 extern	void checkbounds(int *rowp, int *colp);
 extern	void clearent(struct ent *v);
-extern	void clean_crange();
-extern	void clean_frange();
-extern	void clean_range();
+extern	void clean_crange(void);
+extern	void clean_frange(void);
+extern	void clean_range(void);
 extern	void closecol(int arg);
 extern	void closefile(FILE *f, int pid, int rfd);
 extern	void closerow(int r, int numrow);
-extern	void colshow_op();
+extern	void colshow_op(void);
 extern	void copy(struct ent *dv1, struct ent *dv2, struct ent *v1,
 	struct ent *v2);
 extern	void copyent(register struct ent *n, register struct ent *p,
@@ -453,22 +466,22 @@ extern	void deleterow(register int arg);
 extern	void del_range(struct ent *left, struct ent *right);
 extern	void del_abbr(char *abbrev);
 extern	void deraw(int ClearLastLine);
-extern	void diesave();
+extern	void diesave(void);
 extern	void doend(int rowinc, int colinc);
 extern	void doformat(int c1, int c2, int w, int p, int r);
-extern	void dupcol();
-extern	void duprow();
+extern	void dupcol(void);
+extern	void duprow(void);
 extern	void doquery(char *s, char *data, int fd);
 extern	void dostat(int fd);
 extern	void dotick(int tick);
 extern	void editexp(int row, int col);
 extern	void editfmt(int row, int col);
-extern	void edit_mode();
+extern	void edit_mode(void);
 extern	void edits(int row, int col);
 extern	void editv(int row, int col);
 extern	void efree(struct enode *e);
 extern	void erase_area(int sr, int sc, int er, int ec, int ignorelock);
-extern	void erasedb();
+extern	void erasedb(void);
 extern	void eraser(struct ent *v1, struct ent *v2);
 extern	void fgetnum(int r0, int c0, int rn, int cn, int fd);
 extern	void fill(struct ent *v1, struct ent *v2, double start, double inc);
@@ -478,7 +491,7 @@ extern	void fix_frames(int row1, int col1, int row2, int col2,
 	int delta1, int delta2);
 extern	void fix_ranges(int row1, int col1, int row2, int col2,
 	int delta1, int delta2);
-extern	void flush_saved();
+extern	void flush_saved(void);
 extern	void formatcol(int arg);
 extern	void format_cell(struct ent *v1, struct ent *v2, char *s);
 extern	void forwcol(int arg);
@@ -489,63 +502,63 @@ extern	void getfmt(int r0, int c0, int rn, int cn, int fd);
 extern	void getformat(int col, int fd);
 extern	void getnum(int r0, int c0, int rn, int cn, int fd);
 extern	void getstring(int r0, int c0, int rn, int cn, int fd);
-extern	void go_last();
-extern	void goraw();
-extern	void help();
+extern	void go_last(void);
+extern	void goraw(void);
+extern	void help(void);
 extern	void hide_col(int arg);
 extern	void hide_row(int arg);
 extern	void hidecol(int arg);
 extern	void hiderow(int arg);
 extern	void initcolor(int colornum);
-extern	void initkbd();
+extern	void initkbd(void);
 extern	void ins_in_line(int c);
 extern	void ins_string(char *s);
-extern	void insert_mode();
+extern	void insert_mode(void);
 extern	void insertcol(int arg, int delta);
 extern	void insertrow(int arg, int delta);
-extern	void kbd_again();
+extern	void kbd_again(void);
 extern	void label(register struct ent *v, register char *s, int flushdir);
 extern	void let(struct ent *v, struct enode *e);
 extern	void list_colors(FILE *f);
 extern	void list_ranges(FILE *f);
 extern	void lock_cells(struct ent *v1, struct ent *v2);
-extern	void markcell();
+extern	void markcell(void);
 extern	void move_area(int dr, int dc, int sr, int sc, int er, int ec);
 extern	void mover(struct ent *d, struct ent *v1, struct ent *v2);
 extern	void moveto(int row, int col, int lastrow, int lastcol,
 	int cornrow, int corncol);
-extern	void toggle_navigate_mode();
+extern	void toggle_navigate_mode(void);
 extern	void num_search(double n, int firstrow, int firstcol, int lastrow,
 	int lastcol, int errsearch);
 extern	void printfile(char *fname, int r0, int c0, int rn, int cn);
 extern	void pullcells(int to_insert);
-extern	void query(char *s, char *data);
-extern	void read_hist();
+extern	void query(const char *s, char *data);
+extern	void read_hist(void);
 extern	void remember(int save);
-extern	void resetkbd();
-extern	void rowshow_op();
-extern	void scxfree(char *p);
+extern	void resetkbd(void);
+extern	void rowshow_op(void);
+extern	void scxfree(void *p);
 extern	void setauto(int i);
 extern	void setiterations(int i);
 extern	void setorder(int i);
 extern	void showcol(int c1, int c2);
-extern	void showdr();
+extern	void showdr(void);
 extern	void showrow(int r1, int r2);
 extern	void showstring(char *string, int dirflush, int hasvalue, int row,
 	int col, int *nextcolp, int mxcol, int *fieldlenp, int r, int c,
 	struct frange *fr, int frightcols, int flcols, int frcols);
-extern	void signals();
+extern	void signals(void);
 extern	void slet(struct ent *v, struct enode *se, int flushdir);
 extern	void sortrange(struct ent *left, struct ent *right, char *criteria);
-extern	void startshow();
-extern	void startdisp();
-extern	void stopdisp();
+extern	void startshow(void);
+extern	void startdisp(void);
+extern	void stopdisp(void);
 extern	void str_search(char *s, int firstrow, int firstcol, int lastrow,
 	int lastcol, int num);
-extern	void sync_cranges();
-extern	void sync_franges();
-extern	void sync_ranges();
-extern	void sync_refs();
+extern	void sync_cranges(void);
+extern	void sync_franges(void);
+extern	void sync_ranges(void);
+extern	void sync_refs(void);
 extern	void tblprintfile(char *fname, int r0, int c0, int rn, int cn);
 extern	void unlock_cells(struct ent *v1, struct ent *v2);
 extern	void update(int anychanged);
@@ -556,16 +569,14 @@ extern	void write_colors(FILE *f, int indent);
 extern	void write_cranges(FILE *f);
 extern	void write_fd(register FILE *f, int r0, int c0, int rn, int cn);
 extern	void write_franges(FILE *f);
-extern	void write_hist();
+extern	void write_hist(void);
 extern	void write_line(int c);
 extern	void write_ranges(FILE *f);
 extern	void yank_area(int sr, int sc, int er, int ec);
 extern	void yyerror(char *err);
-extern	int yylex();
-extern	int yyparse();
-#ifdef DOBACKUPS
+extern	int yylex(void);
+extern	int yyparse(void);
 extern	int backup_file(char *path);
-#endif
 
 extern	int modflg;
 #if !defined(VMS) && !defined(MSDOS) && defined(CRYPT_PATH)
@@ -601,16 +612,64 @@ extern	int colorneg;
 extern	int colorerr;
 extern	int braille;
 extern	int braillealt;
+extern	int dobackups;
 extern	int loading;
 extern	int getrcqual;
 extern	int tbl_style;
 extern	int rndtoeven;
 extern	char *progname;
+#ifdef TRACE
+extern	FILE *ftrace;
+#endif
 extern	int numeric_field;
 extern	int craction;
 extern	int  pagesize;	/* If nonzero, use instead of 1/2 screen height */
 extern	int rowlimit;
 extern	int collimit;
+#ifdef NCURSES_MOUSE_VERSION
+extern MEVENT mevent;
+#endif
+void gotonote(void);
+void center(int, int, int, int);
+void rjustify(int, int, int, int);
+void ljustify(int, int, int, int);
+void yankcol(int);
+void yankrow(int);
+void list_frames(FILE *);
+void yankr(struct ent *, struct ent *);
+void dogetkey(void);
+void doseval(struct enode *, int, int, int);
+void doeval(struct enode *, char *, int, int, int);
+void getrange(char *, int);
+void getframe(int);
+void add_abbr(char *);
+void repaint(int, int, int, int, int);
+void update(int);
+void doshell(void);
+#ifdef SIGVOID
+void doquit(int);
+void time_out(int);
+void dump_me(int);
+void nopipe(int);
+# ifdef	SIGWINCH
+void winchg(int);
+# endif
+#else
+int doquit();
+int time_out();
+int dump_me();
+int nopipe();
+# ifdef	SIGWINCH
+int winchg();
+# endif
+#endif
+void gohome(void);
+void leftlimit(void);
+void rightlimit(void);
+void gototop(void);
+void gotobottom(void);
+void mouseon(void);
+void mouseoff(void);
 
 #if BSD42 || SYSIII
 
