@@ -12,14 +12,17 @@
  */
 
 
-#include <curses.h>
 #include <time.h>
-#include "sc.h"
+#include <limits.h>
+#include <string.h>
 
 #ifndef MSDOS
 #include <unistd.h>
 #endif
 
+#include "compat.h"
+#include "sc.h"
+
 #ifdef VMS
 extern int VMS_read_raw;   /*sigh*/
     VMS_read_raw = 1;
@@ -33,12 +36,10 @@ extern int VMS_read_raw;   /*sigh*/
 #define nonl()	 (_tty.sg_flags &= ~CRMOD, _pfast = TRUE, stty(_tty_ch, &_tty))
 #endif
 
-void	repaint(int x, int y, int len, int attron, int attroff);
-
 char	under_cursor = ' '; /* Data under the < cursor */
 char	mode_ind = 'i';
 char	search_ind = ' ';
-extern	char    revmsg[];
+extern	char    revmsg[80];
 
 int	lines, cols;
 int	rows, lcols;
@@ -58,7 +59,6 @@ bool	frTooLarge = 0;	/* If set, either too many rows or too many columns
 int	framerows;	/* Rows in current frame */
 int	framecols;	/* Columns in current frame */
 int	rescol = 4;	/* Columns reserved for row numbers */
-extern	int *fwidth;
 extern	int showneed;	/* Causes cells needing values to be highlighted */
 extern	int showexpr;	/* Causes cell exprs to be displayed, highlighted */
 extern	int shownote;	/* Causes cells with attached notes to be highlighted */
@@ -234,13 +234,15 @@ update(int anychanged)		/* did any cell really change in value? */
 	    i = stcol;
 	    lcols = 0;
 	    col = rescol + frcols;
-	    if (fr && stcol >= fr->or_left->col)
+	    if (fr && stcol >= fr->or_left->col) {
 		if (stcol < fr->ir_left->col)
 		    i = fr->or_left->col;
 		else
 		    col += flcols;
-	    for (; (col + fwidth[i] < cols-1 || col_hidden[i] || i < curcol) &&
-		    i < maxcols; i++) {
+	    }
+	    for (; i < maxcols &&
+		    (col + fwidth[i] < cols-1 || col_hidden[i] || i < curcol);
+		    i++) {
 		lcols++;
 		if (fr && i == fr->ir_right->col + 1) {
 		    col -= frcols;
@@ -328,13 +330,15 @@ update(int anychanged)		/* did any cell really change in value? */
 	    i = stcol;
 	    lcols = 0;
 	    col = rescol + frcols;
-	    if (fr && stcol >= fr->or_left->col)
+	    if (fr && stcol >= fr->or_left->col) {
 		if (stcol < fr->ir_left->col)
 		    i = fr->or_left->col;
 		else
 		    col += flcols;
-	    for (; (col + fwidth[i] < cols-1 || col_hidden[i] || i < curcol) &&
-		    i < maxcols; i++) {
+	    }
+	    for (; i < maxcols &&
+		    (col + fwidth[i] < cols-1 || col_hidden[i] || i < curcol);
+		    i++) {
 		lcols++;
 		if (fr && i == fr->ir_right->col + 1) {
 		    col -= frcols;
@@ -377,11 +381,12 @@ update(int anychanged)		/* did any cell really change in value? */
 	    i = strow;
 	    rows = 0;
 	    row = RESROW + fbrows;
-	    if (fr && strow >= fr->or_left->row)
+	    if (fr && strow >= fr->or_left->row) {
 		if (strow < fr->ir_left->row)
 		    i = fr->or_left->row;
 		else
 		    row += ftrows;
+	    }
 	    for (; (row < lines || row_hidden[i] || i < currow) && i < maxrows;
 		    i++) {
 		rows++;
@@ -451,20 +456,22 @@ update(int anychanged)		/* did any cell really change in value? */
 			continue;
 		    row--;
 		}
-		if (fr && strow < fr->or_left->row + ftoprows)
+		if (fr && strow < fr->or_left->row + ftoprows) {
 		    strow = fr->or_left->row + ftoprows;
 		    if (currow < strow)
 			strow = currow;
+		}
 	    }
 	    /* Now pick up the counts again */
 	    i = strow;
 	    rows = 0;
 	    row = RESROW + fbrows;
-	    if (fr && strow >= fr->or_left->row)
+	    if (fr && strow >= fr->or_left->row) {
 		if (strow < fr->ir_left->row)
 		    i = fr->or_left->row;
 		else
 		    row += ftrows;
+	    }
 	    for (; (row < lines || row_hidden[i] || i < currow) && i < maxrows;
 		    i++) {
 		rows++;
@@ -506,7 +513,7 @@ update(int anychanged)		/* did any cell really change in value? */
 		else
 		    color_set(1, NULL);
 		if (*pp) {
-		    if (colorneg && (*pp)->flags & is_valid && (*pp)->v < 0) {
+		    if (colorneg && (*pp)->flags & IS_VALID && (*pp)->v < 0) {
 			if (cr)
 			    color_set(((cr->r_color) % CPAIRS) + 1, NULL);
 			else
@@ -638,13 +645,13 @@ update(int anychanged)		/* did any cell really change in value? */
 	    maxsc = showsc > curcol ? showsc : curcol;
 
 	    if (showtop && !message) {
-		char r[6];
+		char r_[6];
 
-		strcpy(r, coltoa(minsc));
-		strcat(r, ":");
-		strcat(r, coltoa(maxsc));
+		strlcpy(r_, coltoa(minsc), sizeof r_);
+		strlcat(r_, ":", sizeof r_);
+		strlcat(r_, coltoa(maxsc), sizeof r_);
 		(void) clrtoeol();
-		(void) printw("Default range:  %s", r);
+		(void) printw("Default range:  %s", r_);
 	    }
 	} else {
 	    minsr = showsr < currow ? showsr : currow;
@@ -723,8 +730,8 @@ update(int anychanged)		/* did any cell really change in value? */
 	    if ((showrange && (!showneed) && (!showexpr)
 			&& (row >= minsr) && (row <= maxsr)
 			&& (col >= minsc) && (col <= maxsc))
-		    || (showneed && (*pp) && ((*pp)->flags & is_valid) &&
-			(((*pp)->flags & is_strexpr) || !((*pp)->expr)))
+		    || (showneed && (*pp) && ((*pp)->flags & IS_VALID) &&
+			(((*pp)->flags & IS_STREXPR) || !((*pp)->expr)))
 		    || (showexpr && (*pp) && ((*pp)->expr))
 		    || (shownote && (*pp) && ((*pp)->nrow >= 0))) {
 
@@ -747,13 +754,13 @@ update(int anychanged)		/* did any cell really change in value? */
 	    if ((cr = find_crange(row, col)) && color && has_colors())
 		color_set(cr->r_color, NULL);
 
-	    if ((*pp) && (((*pp)->flags & is_changed || FullUpdate) ||
+	    if ((*pp) && (((*pp)->flags & IS_CHANGED || FullUpdate) ||
 		    do_stand)) {
 		if (do_stand) {
-		    (*pp)->flags |= is_changed; 
+		    (*pp)->flags |= IS_CHANGED; 
 		} else {
 		    (void) move(r, c);
-		    (*pp)->flags &= ~is_changed;
+		    (*pp)->flags &= ~IS_CHANGED;
 		}
 
 		/*
@@ -778,7 +785,7 @@ update(int anychanged)		/* did any cell really change in value? */
 		     * Show cell's numeric value:
                      */
 
-		    if ((*pp)->flags & is_valid) {
+		    if ((*pp)->flags & IS_VALID) {
 			char field[FBUFLEN];
 			char *cfmt;
 			int note;
@@ -808,43 +815,59 @@ update(int anychanged)		/* did any cell really change in value? */
 				    precision[col], (*pp)->v, 
 				    field, sizeof(field));
 			}
-			if (strlen(field) > fwidth[col]) {
+			if ((ssize_t)strlen(field) > fwidth[col]) {
 			    for (i = 0; i < fwidth[col]; i++) {
 				if (note) {
+#ifndef NO_ATTR_GET
 				    attr_t attr;
-				    short curcolor;
+				    short curcolor = 0;
 				    if (!i && color && has_colors()) {
-					attr_get(&attr, &curcolor, NULL);
+					/* silence warning */
+					attr_t *attrp = &attr;
+					short *curcolorp = &curcolor;
+					attr_get(attrp, curcolorp, NULL);
 					color_set(4, NULL);
 				    }
+#endif
 				    (void)addch('*');
 				    i++;
+#ifndef NO_ATTR_GET
 				    if (!i && color && has_colors())
 					color_set(curcolor, NULL);
+#endif
 				}
 				(void)addch('*');
 			    }
 			} else {
 			    if (cfmt && *cfmt != ctl('d'))
 				for (i = 0;
-					i < fwidth[col] - strlen(field) - note;
+					i < fwidth[col] -
+					  (ssize_t)strlen(field) - note;
 					i++)
 				    (void)addch(' ');
 			    if (note) {
+#ifndef NO_ATTR_GET
 				attr_t attr;
-				short curcolor;
+				short curcolor = 0;
 				if (color && has_colors()) {
-				    attr_get(&attr, &curcolor, NULL);
+				    /* silence warning */
+				    attr_t *attrp = &attr;
+				    short *curcolorp = &curcolor;
+				    attr_get(attrp, curcolorp, NULL);
 				    color_set(4, NULL);
 				}
+#endif
 				(void)addch('*');
+#ifndef NO_ATTR_GET
 				if (color && has_colors())
 				    color_set(curcolor, NULL);
+#endif
 			    }
 			    (void)addstr(field);
 			    if (cfmt && *cfmt == ctl('d'))
 				for (i = 0;
-					i < fwidth[col] - strlen(field) - note;
+					i < fwidth[col] -
+					  (ssize_t)strlen(field) - note;
 					i++)
 				    (void)addch(' ');
 			}
@@ -856,21 +879,21 @@ update(int anychanged)		/* did any cell really change in value? */
 
 		    if ((*pp)->label) {
 			showstring((*pp)->label,
-				    (*pp)->flags & (is_leftflush|is_label),
-				    (*pp)->flags & is_valid,
+				    (*pp)->flags & (IS_LEFTFLUSH|IS_LABEL),
+				    (*pp)->flags & IS_VALID,
 				    row, col, &nextcol, mxcol, &fieldlen,
 				    r, c, fr, frightcols, flcols, frcols);
 		    } else	/* repaint a blank cell: */
 		    if ((((do_stand || !FullUpdate) &&
-			    ((*pp)->flags & is_changed)) ||
+			    ((*pp)->flags & IS_CHANGED)) ||
 			    (color && has_colors() &&
 			    cr && cr->r_color != 1)) &&
-			    !((*pp)->flags & is_valid) && !(*pp)->label) {
+			    !((*pp)->flags & IS_VALID) && !(*pp)->label) {
 			(void) printw("%*s", fwidth[col], " ");
 		    }
 		} /* else */
 	    } else
-	    if (!*pp && color && has_colors && cr && cr->r_color != 1) {
+	    if (!*pp && color && has_colors() && cr && cr->r_color != 1) {
 		move(r, c);
 		color_set(cr->r_color, NULL);
 		printw("%*s", fwidth[col], " ");
@@ -896,7 +919,7 @@ update(int anychanged)		/* did any cell really change in value? */
 	    else
 		color_set(1, NULL);
 	    if (*pp) {
-		if (colorneg && (*pp)->flags & is_valid && (*pp)->v < 0) {
+		if (colorneg && (*pp)->flags & IS_VALID && (*pp)->v < 0) {
 		    if (cr)
 			color_set(((cr->r_color) % CPAIRS) + 1, NULL);
 		    else
@@ -969,21 +992,21 @@ update(int anychanged)		/* did any cell really change in value? */
 		 * Display string part of cell:
 		 */
 
-		if ((p1->expr) && (p1->flags & is_strexpr)) {
- 		    if (p1->flags & is_label)
+		if ((p1->expr) && (p1->flags & IS_STREXPR)) {
+ 		    if (p1->flags & IS_LABEL)
 			(void) addstr("|{");
 		    else
-			(void) addstr((p1->flags & is_leftflush) ? "<{" : ">{");
+			(void) addstr((p1->flags & IS_LEFTFLUSH) ? "<{" : ">{");
 		    (void) addstr(line);
 		    (void) addstr("} ");	/* and this '}' is for vi % */
 		    printed = 1;
 
 		} else if (p1->label) {
 		    /* has constant label only */
-		    if (p1->flags & is_label)
+		    if (p1->flags & IS_LABEL)
 			(void) addstr("|\"");
 		    else
-			(void) addstr((p1->flags & is_leftflush) ? "<\"" : ">\"");
+			(void) addstr((p1->flags & IS_LEFTFLUSH) ? "<\"" : ">\"");
 		    (void) addstr(p1->label);
 		    (void) addstr("\" ");
 		    printed = 1;
@@ -993,10 +1016,10 @@ update(int anychanged)		/* did any cell really change in value? */
 		 * Display value part of cell:
 		 */
 
-		if (p1->flags & is_valid) {
+		if (p1->flags & IS_VALID) {
 		    /* has value or num expr */
-		    if ((!(p1->expr)) || (p1->flags & is_strexpr))
-			(void) sprintf(line, "%.15g", p1->v);
+		    if ((!(p1->expr)) || (p1->flags & IS_STREXPR))
+			snprintf(line, sizeof line, "%.15g", p1->v);
 
 		    (void) addch('[');
 		    (void) addstr(line);
@@ -1008,7 +1031,7 @@ update(int anychanged)		/* did any cell really change in value? */
 	    if (!printed)
 		(void) addstr("[]");
 	    /* Display if cell is locked */
-	    if (p1 && p1->flags&is_locked)
+	    if (p1 && p1->flags&IS_LOCKED)
 		(void) addstr(" locked");
 	}
 	if (braille)
@@ -1087,12 +1110,12 @@ yyerror(char *err)
     if (usecurses) {
 	if (seenerr) return;
 	seenerr++;
-	(void) move(1, 0);
-	(void) clrtoeol();
-	(void) printw("%s: %.*s<=%s", err, linelim, line, line + linelim);
+	move(1, 0);
+	clrtoeol();
+	printw("%s: %.*s<=%s", err, (int)linelim, line, line + linelim);
     } else
-	(void) fprintf(stderr, "%s: %.*s<=%s\n", err, linelim, line,
-		       line + linelim);
+	fprintf(stderr, "%s: %.*s<=%s\n", err, (int)linelim, line,
+	  line + linelim);
 }
 
 #ifdef XENIX2_3
@@ -1100,7 +1123,7 @@ struct termio tmio;
 #endif
 
 void
-startdisp()
+startdisp(void)
 {
 #if sun
     int	 fd;
@@ -1177,7 +1200,7 @@ startdisp()
 }
 
 void
-stopdisp()
+stopdisp(void)
 {
 #ifndef MSDOS
     if (usecurses) {
@@ -1197,8 +1220,7 @@ stopdisp()
 #ifdef VMS
 
 void
-goraw()
-{
+goraw(void) {
     if (usecurses) {
 	VMS_read_raw = 1;
 	if (color && has_colors())
@@ -1224,16 +1246,15 @@ deraw(int ClearLastLine)
 
 #else /* VMS */
 void
-goraw()
-{
+goraw(void) {
     if (usecurses) {
-#if SYSV2 || SYSV3
+#ifdef HAVE_FIXTERM
 	fixterm();
-#else /* SYSV2 || SYSV3 */
+#else
 	cbreak();
 	nonl();
 	noecho ();
-#endif /* SYSV2 || SYSV3 */
+#endif
 	kbd_again();
 	if (color && has_colors())
 	    bkgdset(COLOR_PAIR(1) | ' ');
@@ -1253,7 +1274,7 @@ deraw(int ClearLastLine)
 	    (void) clrtoeol();
 	    (void) refresh();
 	}
-#if SYSV2 || SYSV3
+#ifdef HAVE_RESETTERM
 	resetterm();
 #else
 	nocbreak();
