@@ -5,23 +5,18 @@
  */
 
 #include <sys/types.h>
-#ifdef BSD42
-#include <strings.h>
-#else
-#ifndef SYSIII
+#include <sys/wait.h>
 #include <string.h>
-#endif
-#endif
-
 #include <signal.h>
-#include <curses.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include "sc.h"
-
+#include <unistd.h>
+#include <limits.h>
 #if defined(REGCOMP)
 #include <regex.h>
 #endif
+#include "compat.h"
+#include "sc.h"
 #if defined(RE_COMP)
 extern char *re_comp(char *s);
 extern char *re_exec(char *s);
@@ -31,62 +26,57 @@ char *regcmp();
 char *regex();
 #endif
 
-void doshell();
-void gohome();
-void leftlimit();
-void rightlimit();
-void gototop();
-void gotobottom();
-
 #define istext(a) (isalnum(a) || ((a) == '_'))
 
-#define bool	int
-#define true	1
-#define false	0
-
-static void append_line();
-static void back_hist();
+static void append_line(void);
+static void back_hist(void);
 static int  back_line(int arg);
 static int  back_word(int arg, int big_word);
-static void back_space();
+static void back_space(void);
 static void change_case(int arg);
-static void col_0();
-static void cr_line();
+static void col_0(void);
+static void cr_line(void);
 static void del_in_line(int arg, int back_null);
-static void del_to_end();
-static void doabbrev();
-static void dogoto();
-static void dotab();
-static void dotcmd();
+static void del_to_end(void);
+static void doabbrev(void);
+static void dogoto(void);
+static void dotab(void);
+static void dotcmd(void);
 static int  find_char(int arg, int n);
-static void for_hist();
+static void for_hist(void);
 static int  for_line(int arg, int stop_null);
 static int  for_word(int arg, int end_word, int big_word, int stop_null);
 static int  istart;
-static void last_col();
-static void match_paren();
+static void last_col(void);
+static void match_paren(void);
 static void readhistfile(FILE *fp);
-static void rep_char();
+static void rep_char(void);
 static void replace_in_line(int c);
-static void replace_mode();
-static void restore_it();
+static void replace_mode(void);
+static void restore_it(void);
 static void savedot(int c);
-static void save_hist();
+static void save_hist(void);
 static void search_again(bool reverse);
-static void search_hist();
+static void search_hist(void);
 static void search_mode(char sind);
-static void stop_edit();
+static void stop_edit(void);
 static int  to_char(int arg, int n);
 static void u_save(int c);
 static void yank_cmd(int delete, int change);
 static void yank_chars(register int first, register int last, int delete);
+static int get_motion(int);
+int vigetch(void);
+#ifdef NCURSES_MOUSE_VERSION
+void mouse_set_pos(int);
+#endif
 
 extern int framerows;		/* Rows in current frame */
 extern char mode_ind;		/* Mode indicator */
 extern char search_ind;		/* Search indicator */
 extern int lcols;		/* Spreadsheet Column the cursor was in last */
-char	*completethis = NULL;
-int	search_dir;		/* Search direction:  forward = 0; back = 1 */
+static char	*completethis = NULL;
+static int	search_dir;		/* Search direction:  forward = 0; back = 1 */
+char histfile[PATHLEN] = "~/.sc_history";
 
 /* values for mode below */
 
@@ -100,7 +90,7 @@ int	search_dir;		/* Search direction:  forward = 0; back = 1 */
 #define	DOTLEN		200
 
 static int mode = INSERT_MODE;
-struct	hist {
+static struct	hist {
 	unsigned int	len;
 	char		*histline;
 } history[HISTLEN + 1];
@@ -121,6 +111,7 @@ static char	*last_search = NULL;
 #endif
 #endif
 static char	*undo_line = NULL;
+static unsigned	undolen = 0;
 static int	undo_lim;
 static char	dotb[DOTLEN];
 static int	doti = 0;
@@ -138,7 +129,7 @@ write_line(int c)
     struct frange *fr;
     struct crange *cr;
 
-    error("");
+    CLEAR_LINE;
     if (c != ctl('i')) completethis = NULL;
     if (mode == EDIT_MODE) {
 	nosavedot = 0;
@@ -149,7 +140,8 @@ write_line(int c)
 	case (ctl('m')):	if (search_ind != ' ')
 				    search_hist();
 				else
-				    cr_line();				break;
+				    cr_line();
+			        break;
 	case 'v':
 	case (ctl('v')):	toggle_navigate_mode();			break;
 	case ESC:	stop_edit();					break;
@@ -240,7 +232,20 @@ write_line(int c)
 	case KEY_DC:
 	case 'x':	u_save(c); del_in_line(arg, 1);			break;
 	case 'y':	yank_cmd(0, 0);					break;
-	default:							break;
+#ifdef NCURSES_MOUSE_VERSION
+	case KEY_MOUSE:
+	    if (getmouse(&mevent) != OK || mevent.y)
+		break;
+	    if (mevent.bstate & BUTTON1_CLICKED)
+		mouse_set_pos(0);
+	    else if (mevent.bstate & BUTTON1_PRESSED)
+		mouse_set_pos(1);
+	    else if (mevent.bstate & BUTTON1_RELEASED)
+		mouse_set_pos(2);
+	    break;
+#endif
+	default:
+	    break;
 	}
     } else if (mode == INSERT_MODE) { 
 	if (c == (ctl('m')))
@@ -255,7 +260,8 @@ write_line(int c)
 	case (ctl('v')):	toggle_navigate_mode();			break;
 	case KEY_LEFT:
 	case (ctl('b')):	if (numeric_field) {
-				    if (linelim == strlen(line) &&
+				    if (linelim > 0 &&
+				            (size_t)linelim == strlen(line) &&
 					    (line[linelim - 1] == '+' ||
 					     line[linelim - 1] == '-')) {
 					toggle_navigate_mode();
@@ -273,7 +279,8 @@ write_line(int c)
 				}   break;
 	case KEY_RIGHT:
 	case (ctl('f')):	if (numeric_field) {
-				    if (linelim == strlen(line) &&
+				    if (linelim > 0 &&
+				            (size_t)linelim == strlen(line) &&
 					    (line[linelim - 1] == '+' ||
 					     line[linelim - 1] == '-')) {
 					toggle_navigate_mode();
@@ -291,7 +298,8 @@ write_line(int c)
 				}   break;
 	case KEY_DOWN:
 	case (ctl('n')):	if (numeric_field) {
-				    if (linelim == strlen(line) &&
+				    if (linelim > 0 &&
+				            (size_t)linelim == strlen(line) &&
 					    (line[linelim - 1] == '+' ||
 					     line[linelim - 1] == '-')) {
 					toggle_navigate_mode();
@@ -309,7 +317,8 @@ write_line(int c)
 				}   break;
 	case KEY_UP:
 	case (ctl('p')):	if (numeric_field) {
-				    if (linelim == strlen(line) &&
+				    if (linelim > 0 &&
+				            (size_t)linelim == strlen(line) &&
 					    (line[linelim - 1] == '+' ||
 					     line[linelim - 1] == '-')) {
 					toggle_navigate_mode();
@@ -335,6 +344,18 @@ write_line(int c)
 	 * character in the line
 	 */
 	case '\035':		if (linelim > 0) doabbrev();		break;
+#ifdef NCURSES_MOUSE_VERSION
+	case KEY_MOUSE:
+	    if (getmouse(&mevent) != OK || mevent.y)
+		break;
+	    if (mevent.bstate & BUTTON1_CLICKED)
+		mouse_set_pos(0);
+	    else if (mevent.bstate & BUTTON1_PRESSED)
+		mouse_set_pos(1);
+	    else if (mevent.bstate & BUTTON1_RELEASED)
+		mouse_set_pos(2);
+	    break;
+#endif
 	default:		ins_in_line(c);				break;
 	}
     } else if (mode == SEARCH_MODE) {
@@ -354,7 +375,8 @@ write_line(int c)
 	savedot(c);
 	switch (c) {
 	case KEY_BACKSPACE:
-	case (ctl('h')):	if (linelim > strlen(undo_line))
+	case (ctl('h')):	if (linelim >= 0 &&
+				  (size_t)linelim > strlen(undo_line))
 				    back_space();
 				else {
 				    linelim = back_line(1);
@@ -371,7 +393,7 @@ write_line(int c)
 	case (ctl('i')):	if (!showrange) {
 				    toggle_navigate_mode();
 				    startshow();
-				} else if (linelim == strlen(line) &&
+				} else if ((size_t)linelim == strlen(line) &&
 					(line[linelim - 1] == '+' ||
 					line[linelim - 1] == '-' ||
 					(line[linelim - 1] == ' ' &&
@@ -394,7 +416,7 @@ write_line(int c)
 	case '-':		if (!showrange) {
 				    ins_string(v_name(currow, curcol));
 				    ins_in_line(c);
-				} else if (linelim == strlen(line) &&
+				} else if ((size_t)linelim == strlen(line) &&
 					(line[linelim - 1] == '+' ||
 					line[linelim - 1] == '-' ||
 					(line[linelim - 1] == ' ' &&
@@ -420,8 +442,8 @@ write_line(int c)
 				    struct ent *p = *ATBL(tbl, currow, curcol);
 				    char temp[100];
 
-				    if (p && p->flags & is_valid) {
-					(void) sprintf(temp, "%.*f",
+				    if (p && p->flags & IS_VALID) {
+					snprintf(temp, sizeof temp, "%.*f",
 						precision[curcol], p->v);
 					ins_string(temp);
 					toggle_navigate_mode();
@@ -507,12 +529,12 @@ write_line(int c)
 	case '#':		gotobottom();				break;
 	case 'o':		if (showrange) {
 				    int r = currow;
-				    int c = curcol;
+				    int cc = curcol;
 
 				    currow = showsr;
 				    showsr = r;
 				    curcol = showsc;
-				    showsc = c;
+				    showsc = cc;
 				    rowsinrange = 1;
 				    colsinrange = fwidth[curcol];
 				} 					break;
@@ -578,8 +600,7 @@ write_line(int c)
 }
 
 void
-edit_mode()
-{
+edit_mode(void) {
     mode_ind = 'e';
     mode = EDIT_MODE;
     if (linelim < 0)	/* -1 says stop editing, ...so we still aren't */
@@ -589,7 +610,7 @@ edit_mode()
 }
 
 void
-insert_mode()
+insert_mode(void)
 {
     mode_ind = 'i';
     mode = INSERT_MODE;
@@ -619,14 +640,13 @@ search_mode(char sind)
 }
 
 static void
-replace_mode()
-{
+replace_mode(void) {
     mode_ind = 'R';
     mode = REP_MODE;
 }
 
 void
-toggle_navigate_mode()
+toggle_navigate_mode(void)
 {
     static char prev_mode = NAVIGATE_MODE;
     int limtmp;
@@ -660,17 +680,16 @@ toggle_navigate_mode()
 }
 
 void
-dotab()
-{
+dotab(void) {
     static struct range *firstmatch;
     static struct range *lastmatch;
     static struct range *nextmatch;
     int len;
 
-    if (linelim > 0 && isalnum(line[linelim-1]) || line[linelim-1] == '_' ||
+    if ((linelim > 0 && isalnum((int)line[linelim-1])) || line[linelim-1] == '_' ||
 	    (completethis && line[linelim-1] == ' ')) {
 	if (!completethis) {
-	    for (completethis = line + linelim - 1; isalnum(*completethis) ||
+	    for (completethis = line + linelim - 1; isalnum((int)*completethis) ||
 		    *completethis == '_'; completethis--) /* */;
 	    completethis++;
 	    len = line + linelim - completethis;
@@ -701,7 +720,7 @@ dotab()
 
 /* show the current range (see ^I), we are moving around to define a range */
 void
-startshow()
+startshow(void)
 {
     showrange = 1;
     showsr = currow;
@@ -712,10 +731,9 @@ startshow()
 /* insert the range we defined by moving around the screen, see startshow() */
 
 void
-showdr()
+showdr(void)
 {
     int			minsr, minsc, maxsr, maxsc;
-    char		*p;
     char		r[12];
     struct frange	*fr = find_frange(currow, curcol);
 
@@ -726,7 +744,7 @@ showdr()
 	maxsc = fr ? fr->or_right->col : maxcols;
 
 	toggle_navigate_mode();
-	sprintf(r, "%d:%d", minsr, maxsr);
+	snprintf(r, sizeof r, "%d:%d", minsr, maxsr);
 	ins_string(r);
     } else if (showrange == SHOWCOLS) {
 	minsr = 0;
@@ -734,9 +752,9 @@ showdr()
 	maxsr = maxrows;
 	maxsc = showsc > curcol ? showsc : curcol;
 
-	strcpy(r, coltoa(minsc));
-	strcat(r, ":");
-	strcat(r, coltoa(maxsc));
+	strlcpy(r, coltoa(minsc), sizeof r);
+	strlcat(r, ":", sizeof r);
+	strlcat(r, coltoa(maxsc), sizeof r);
 
 	toggle_navigate_mode();
 	ins_string(r);
@@ -774,11 +792,9 @@ savedot(int c)
     }
 }
 
-static int dotcalled = 0;
-
 static void
-dotcmd()
-{
+dotcmd(void) {
+    static int dotcalled = 0;
     int c;
 
     if (dotcalled)	/* stop recursive calling of dotcmd() */
@@ -801,8 +817,7 @@ dotcmd()
 }
 
 int
-vigetch()
-{
+vigetch(void) {
     int c;
 
     if (do_dot) {
@@ -823,14 +838,12 @@ vigetch()
 static void
 u_save(int c)
 {
-    static unsigned	undolen = 0;
-
     if (strlen(line)+1 > undolen) {
     	undolen = strlen(line)+40;
 
 	undo_line = scxrealloc(undo_line, undolen);
     }
-    (void) strcpy(undo_line, line);
+    strlcpy(undo_line, line, undolen);
 
     undo_lim = linelim;
 
@@ -844,8 +857,7 @@ u_save(int c)
 
 /* Restores the current line saved by u_save() */
 static void
-restore_it()
-{
+restore_it(void) {
     static char		*tempc = NULL;
     static unsigned	templen = 0;
     int			tempi;
@@ -858,21 +870,20 @@ restore_it()
 	tempc = scxrealloc(tempc, templen);
     }
 
-    strcpy(tempc, line);
+    strlcpy(tempc, line, templen);
     tempi = linelim;
-    (void) strcpy(line, undo_line);
+    strlcpy(line, undo_line, sizeof line);
     linelim = undo_lim;
-    strcpy(undo_line, tempc);
+    strlcpy(undo_line, tempc, undolen);
     undo_lim = tempi;
 }
 
 /* This command stops the editing process. */
 static void
-stop_edit()
-{
+stop_edit(void) {
     if (search_ind != ' ') {
 	search_ind = ' ';
-	(void) strcpy(line, history[0].histline);
+	strlcpy(line, history[0].histline, sizeof line);
 	write_line('G');
     } else {
 	showrange = 0;
@@ -890,18 +901,18 @@ stop_edit()
  * the last character of the line.
  */
 static int
-for_line(int arg, int stop_null)
+for_line(int a, int stop_null)
 {
-    int cpos = linelim;
+    ssize_t cpos = linelim;
 
     if (linelim < 0)
 	return (linelim);
-    else if (linelim + arg <= strlen(line))
-	cpos += arg;
+    else if (a >= 0 && (size_t)(linelim + a) <= strlen(line))
+	cpos += a;
     else
 	cpos = strlen(line);
 
-    if (cpos == strlen(line) && cpos > 0 && !stop_null)
+    if (cpos > 0 && (size_t)cpos == strlen(line) && !stop_null)
 	return (cpos - 1);
     else
 	return (cpos);
@@ -912,14 +923,14 @@ for_line(int arg, int stop_null)
  */
 
 static int
-for_word(int arg, int end_word, int big_word, int stop_null)
+for_word(int a, int end_word, int big_word, int stop_null)
 {
     register int c;
-    register int cpos;
+    ssize_t cpos;
 
     cpos = linelim;
 
-    while (cpos < strlen(line) && arg--) {
+    while (cpos >= 0 && (size_t)cpos < strlen(line) && a--) {
 	if (end_word)
 	    cpos++;
 
@@ -935,7 +946,7 @@ for_word(int arg, int end_word, int big_word, int stop_null)
 	if (big_word)
 	    while ((c = line[cpos]) && c != ' ')
 		cpos++;
-	else if (istext(line[cpos])) {
+	else if (istext((int)line[cpos])) {
 	    while ((c = line[cpos]) && istext(c)) 
 		cpos++;
 	} else {
@@ -956,30 +967,30 @@ for_word(int arg, int end_word, int big_word, int stop_null)
 }
 
 static int
-back_line(int arg)
+back_line(int a)
 {
-    if (linelim > arg)
-        return (linelim - arg);
+    if (linelim > a)
+        return (linelim - a);
     else
 	return (0);
 }
 
 static int
-back_word(int arg, int big_word)
+back_word(int a, int big_word)
 {
     register int c;
     register int cpos;
 
     cpos = linelim;
 
-    while (cpos > 0 && arg--) {
+    while (cpos > 0 && a--) {
 	if (line[cpos] == ' ') {
 	    /* Skip white space */
 	    while (cpos > 0 && line[cpos] == ' ')
 		--cpos;
 	} else if (cpos > 0 && (line[cpos-1] == ' ' 
-			|| ( istext(line[cpos]) && !istext(line[cpos-1]))
-			|| (!istext(line[cpos]) &&  istext(line[cpos-1])))) {
+			|| ( istext((int)line[cpos]) && !istext((int)line[cpos-1]))
+			|| (!istext((int)line[cpos]) &&  istext((int)line[cpos-1])))) {
 	    /* Started on the first char of a word - back up to prev. word */
 	    --cpos;
 	    while (cpos > 0 && line[cpos] == ' ')
@@ -990,7 +1001,7 @@ back_word(int arg, int big_word)
 	if (big_word)
 	    while (cpos > 0 && (c = line[cpos]) && c != ' ')
 		--cpos;
-	else if (istext(line[cpos])) {
+	else if (istext((int)line[cpos])) {
 	    while (cpos > 0 && (c = line[cpos]) && istext(c)) 
 		--cpos;
 	} else {
@@ -1012,20 +1023,20 @@ back_word(int arg, int big_word)
  * line terminator.  Otherwise, don't.
  */
 static void
-del_in_line(int arg, int back_null)
+del_in_line(int a, int back_null)
 {
     register int len, i;
 
     if (linelim >= 0) {
 	len = strlen(line);
-	if (arg > len - linelim)
-	    arg = len - linelim;
+	if (a > len - linelim)
+	    a = len - linelim;
 	if (linelim == len && linelim > 0)
 	    linelim--;
-	strncpy(putbuf, line + linelim, arg);
-	putbuf[arg] = '\0';
+	strlcpy(putbuf, line + linelim, a);
+	putbuf[a] = '\0';
 	for (i = linelim; i < len; i++)
-	    line[i] = line[i+arg];
+	    line[i] = line[i+a];
     }
     if (back_null && linelim > 0 && line[linelim] == '\0')
 	--linelim;
@@ -1065,35 +1076,41 @@ ins_string(char *s)
 }
 
 void
-doabbrev()
-{
+doabbrev(void) {
     int len, pos;
     struct abbrev *a;
     struct abbrev *prev;
-    
-    if (!(isalpha(line[linelim-1]) || isdigit(line[linelim-1]) ||
-	    line[linelim-1] == '_') || !(mode == INSERT_MODE ||
-	    mode == SEARCH_MODE) || istart >= linelim)
+
+    if (istart < 0 || linelim < 2)
+	return;
+
+    if (!(isalpha((int)line[linelim - 1]) || isdigit((int)line[linelim - 1]) ||
+      line[linelim - 1] == '_') || !(mode == INSERT_MODE ||
+      mode == SEARCH_MODE) || istart >= linelim)
 	return;
 
     pos = linelim - 2;
-    if (isalpha(line[pos]) || isdigit(line[pos]) || line[pos] == '_') {
+    if (isalpha((int)line[pos]) || isdigit((int)line[pos]) || line[pos] == '_')
+    {
 	for (; pos >= istart; pos--)
-	    if (!(isalpha(line[pos]) || isdigit(line[pos]) || line[pos] == '_'))
+	    if (!(isalpha((int)line[pos]) || isdigit((int)line[pos]) ||
+	      line[pos] == '_'))
 		break;
     } else if (line[pos] != ' ')
 	for (; pos >= istart; pos--)
-	    if (isalpha(line[pos]) || isdigit(line[pos]) || line[pos] == '_' ||
-		    line[pos] == ' ')
+	    if (isalpha((int)line[pos]) || isdigit((int)line[pos]) || line[pos]
+	      == '_' || line[pos] == ' ')
 		break;
     pos++;
 
     if (istart && pos == istart) {
-	if (isalpha(line[pos]) || isdigit(line[pos]) || line[pos] == '_') {
-	    if (isalpha(line[--pos]) || isdigit(line[pos]) || line[pos] == '_')
+	if (isalpha((int)line[pos]) || isdigit((int)line[pos]) || line[pos] ==
+	  '_') {
+	    if (isalpha((int)line[--pos]) || isdigit((int)line[pos]) ||
+	      line[pos] == '_')
 		return;
 	} else {
-	    if (!(isalpha(line[--pos]) || isdigit(line[pos]) ||
+	    if (!(isalpha((int)line[--pos]) || isdigit((int)line[pos]) ||
 		    line[pos] == '_' || line[pos] == ' '))
 		return;
 	}
@@ -1111,8 +1128,7 @@ doabbrev()
 }
 
 static void
-append_line()
-{
+append_line(void) {
     register int i;
 
     i = linelim;
@@ -1122,24 +1138,22 @@ append_line()
 }
 
 static void
-change_case(arg)
-{
+change_case(int a) {
     if (linelim < 0) {
     	linelim = 0;
 	*line = '\0';
     }
-    while (arg--) {
-	if (islower(line[linelim]))
-	    line[linelim] = toupper(line[linelim]);
-	else if (isupper(line[linelim]))
-	    line[linelim] = tolower(line[linelim]);
+    while (a--) {
+	if (islower((int)line[linelim]))
+	    line[linelim] = toupper((int)line[linelim]);
+	else if (isupper((int)line[linelim]))
+	    line[linelim] = tolower((int)line[linelim]);
 	linelim = for_line(1, 0);
     }
 }
 
 static void
-rep_char()
-{
+rep_char(void) {
     int c;
 
     if (linelim < 0) {
@@ -1173,8 +1187,7 @@ replace_in_line(int c)
 }
 
 static void
-back_space()
-{
+back_space(void) {
     if (linelim == 0)
 	return;
 
@@ -1194,7 +1207,7 @@ back_space()
  * `ce', just as in vi.  Setting change to 0 causes `w' to act as usual.
  */
 
-int
+static int
 get_motion(int change)
 {
     int c;
@@ -1261,25 +1274,23 @@ yank_chars(register int first, register int last, int delete)
     }
 
     linelim = first;
-    strncpy(putbuf, line + first, last - first);
+    strlcpy(putbuf, line + first, last - first);
     putbuf[last - first] = '\0';
     if (delete)
 	memmove(line + first, line + last, strlen(line + last) + 1);
 }
 
 static void
-del_to_end()
-{
+del_to_end(void) {
     if (linelim < 0)
 	return;
-    strcpy(putbuf, line + linelim);
+    strlcpy(putbuf, line + linelim, sizeof putbuf);
     line[linelim] = '\0';
     linelim = back_line(1);
 }
 
 static void
-cr_line()
-{
+cr_line(void) {
     struct frange *fr;
 
     ins_in_line(0);
@@ -1371,7 +1382,7 @@ cr_line()
 }
 
 void
-doshell()
+doshell(void)
 {
     /*
     *  "! command"  executes command
@@ -1395,9 +1406,9 @@ doshell()
     (void) fgets(cmd, MAXCMD, stdin);
     cmd[strlen(cmd) - 1] = '\0';	/* clobber \n */
     if (strcmp(cmd,"!") == 0)		/* repeat? */
-	(void) strcpy(cmd, lastcmd);
+	strlcpy(cmd, lastcmd, sizeof cmd);
     else
-	(void) strcpy(lastcmd, cmd);
+	strlcpy(lastcmd, cmd, sizeof lastcmd);
 
     if (modflg) {
 	(void) puts("[No write since last change]");
@@ -1407,9 +1418,9 @@ doshell()
     if (!(pid = fork())) {
 	(void) signal(SIGINT, SIG_DFL);  /* reset */
 	if (strlen(cmd))
-	    (void) execl(shl, shl, "-c", cmd, NULL);
+	    execl(shl, shl, "-c", cmd, (char *)NULL);
 	else
-	    (void) execl(shl, shl, NULL);
+	    execl(shl, shl, (char *)NULL);
 	exit (-127);
     }
 
@@ -1427,8 +1438,7 @@ doshell()
 /* History functions */
 
 static void
-save_hist()
-{
+save_hist(void) {
     if (!lasthist || strcmp(history[lasthist].histline, line)) {
 	if (lasthist < 0)
 	    lasthist = 1;
@@ -1443,7 +1453,7 @@ save_hist()
 	    history[lasthist].histline = scxrealloc(history[lasthist].histline,
 		    history[lasthist].len);
 	}
-	(void) strcpy(history[lasthist].histline, line);
+	strlcpy(history[lasthist].histline, line, history[lasthist].len);
 	histsessionnew++;
     }
     if (history[0].histline) {
@@ -1455,8 +1465,7 @@ save_hist()
 }
 
 static void
-for_hist()
-{
+for_hist(void) {
     if (histp == 0) {
 	last_col();
     	return;
@@ -1468,25 +1477,25 @@ for_hist()
 	histp = histp % endhist + 1;
 
     if (lasthist >= 0) {
-	(void) strcpy(line, history[histp].histline);
+	strlcpy(line, history[histp].histline, sizeof line);
 	last_col();
     }
-    if (histp)
+    if (histp) {
 	error("History line %d", endhist - lasthist + histp);
-    else
-	error("");
+    } else {
+	CLEAR_LINE;
+    }
 }
 
 static void
-back_hist()
-{
+back_hist(void) {
     if (histp == 0) {
 	if (history[0].len < strlen(line) + 1) {
 	    history[0].len = strlen(line) + 40;
 	    history[0].histline = scxrealloc(history[0].histline,
 		    history[0].len);
 	}
-	(void) strcpy(history[0].histline, line);
+	strlcpy(history[0].histline, line, history[0].len);
 
 	if (lasthist >= 0)
 	    histp = lasthist;
@@ -1497,18 +1506,18 @@ back_hist()
 	histp--;
 
     if (lasthist >= 0) {
-    	(void) strcpy(line, history[histp].histline);
+    	strlcpy(line, history[histp].histline, sizeof line);
 	last_col();
     }
-    if (histp)
+    if (histp) {
 	error("History line %d", endhist - lasthist + histp);
-    else
-	error("");
+    } else {
+	CLEAR_LINE;
+    }
 }
 
 static void
-search_hist()
-{
+search_hist(void) {
 #ifdef RECOMP
     char	*tmp;
 #endif
@@ -1522,26 +1531,24 @@ search_hist()
 	return;
     }
 
-#ifdef REGCOMP
+#if defined REGCOMP
     if (last_search)
 	regfree(last_search);
     else
 	last_search = &preg;
     if ((errcode = regcomp(last_search, line, REG_EXTENDED))) {
-	char *tmp = scxmalloc((size_t)160);
+	char *tmp = scxmalloc(160);
 	regerror(errcode, last_search, tmp, sizeof(tmp));
 	error(tmp);
 	scxfree(tmp);
 	return;
     }
-#else
-#ifdef RE_COMP
+#elif defined RE_COMP
     if ((tmp = re_comp(line)) != NULL) {
 	error(tmp);
 	return;
     }
-#else
-#ifdef REGCMP
+#elif defined REGCMP
     free(last_search);
     if ((last_search = regcmp(line, NULL)) == NULL) {
 	error("Invalid search string");
@@ -1552,11 +1559,9 @@ search_hist()
     	lastsrchlen = strlen(line) + 40;
 	last_search = scxrealloc(last_search, lastsrchlen);
     }
-    (void) strcpy(last_search, line);
+    strlcpy(last_search, line, lastsrchlen);
 #endif
-#endif
-#endif
-    (void) strcpy(line, history[0].histline);
+    strlcpy(line, history[0].histline, sizeof line);
     search_again(false);
     if (mode != EDIT_MODE) edit_mode();
     search_ind = ' ';
@@ -1566,37 +1571,35 @@ static void
 search_again(bool reverse)
 {
     int prev_match;
-    int found_it;
+    int found_it = 0;
 #if !defined(REGCOMP) && !defined(RE_COMP) && !defined(REGCMP)
     char *look_here;
     int do_next;
 #endif
 
-#ifdef REGCOMP
-    if ((last_search == NULL))
+#if defined REGCOMP
+    if (last_search == NULL)
 	return;
-#else
-#ifndef RE_COMP
-    if ((last_search == NULL) || (*last_search == '\0'))
+#elif !defined(RE_COMP)
+    if (last_search == NULL || *last_search == '\0')
 	return;
-#endif
 #endif
     prev_match = histp > 0 ? histp : 0;
-    error("");
+    CLEAR_LINE;
 
     do {
 	if (lasthist > 0) {
 	    if (!(search_dir ^ reverse) && histp != lasthist)
 		if (histp <= 0) {
 		    histp = ((lasthist + 1) % endhist);
-		    (void) strcpy(line, history[histp].histline);
+		    strlcpy(line, history[histp].histline, sizeof line);
 		} else
 		    for_hist();
 	    else if ((search_dir ^ reverse) && histp != ((lasthist + 1) % endhist))
 		back_hist();
 	    else {
 		histp = 0;
-		(void) strcpy(line, history[0].histline);
+		strlcpy(line, history[0].histline, sizeof line);
 		last_col();
 	    }
 	} else
@@ -1610,20 +1613,19 @@ search_again(bool reverse)
 	if (histp <= 0) {
 	    if (search_dir ^ reverse)
 		back_hist();
-	    else
+	    else {
 		histp = ((lasthist + 1) % endhist);
-		(void) strcpy(line, history[histp].histline);
+		strlcpy(line, history[histp].histline, sizeof line);
+	    }
 	}
 	found_it = 0;
-#ifdef REGCOMP
+#if defined REGCOMP
 	if (regexec(last_search, line, 0, NULL, 0) == 0)
 	    found_it++;
-#else
-#ifdef RE_COMP
+#elif defined RE_COMP
 	if (re_exec(line) != 0)
 	    found_it++;
-#else
-#ifdef REGCMP
+#elif defined REGCMP
 	if (regex(last_search, line) != NULL)
 	    found_it++;
 #else
@@ -1639,28 +1641,28 @@ search_again(bool reverse)
 	    else
 		do_next++;
 	}
-#endif
-#endif
 #endif
 	if (histp == prev_match)
 	    break;
     } while (!found_it);
-    if (found_it)
+    if (found_it) {
 	error("History line %d", endhist - lasthist + histp);
-    else
+    } else {
 	error("No matches found");
+    }
     edit_mode();
     linelim = strlen(line) - 1;
 }
 
-#if !defined(MSDOS) && defined HISTORY_FILE
+#if !defined(MSDOS)
 void
-write_hist()
+write_hist(void)
 {
     int i;
     FILE *fp, *tmpfp = NULL;
-    char histfile[PATHLEN];
 
+    if (!*histfile)
+	return;
     if (histsessionnew < HISTLEN) {
 	/* write the new history for this session to a tmp file */
 	tmpfp = tmpfile();
@@ -1680,8 +1682,8 @@ write_hist()
     }
 
     /* now write to whole lot out to the proper save file */
-    strcpy(histfile, HISTORY_FILE);
-    if (findhome(histfile) && (fp = fopen(histfile, "w")) != NULL) {
+    if (findhome(histfile, sizeof histfile) && (fp = fopen(histfile, "w"))
+      != NULL) {
 	for (i = 1; i <= endhist; i++) {
 	    lasthist = lasthist % endhist + 1;
 	    if (history[lasthist].len > 40)
@@ -1694,6 +1696,8 @@ write_hist()
 static void
 readhistfile(FILE *fp)
 {
+    if (!*histfile)
+	return;
     while (fgets(line, FBUFLEN, fp)) {
 	line[strlen(line)-1] = '\0'; /* chop the \n */
 	save_hist();
@@ -1702,13 +1706,14 @@ readhistfile(FILE *fp)
 }
 
 void
-read_hist()
+read_hist(void)
 {
     FILE *fp;
-    char histfile[PATHLEN];
 
-    strcpy(histfile, HISTORY_FILE);
-    if (findhome(histfile) && (fp = fopen(histfile, "r")) != NULL)
+    if (!*histfile)
+	return;
+    if (findhome(histfile, sizeof histfile) && (fp = fopen(histfile, "r"))
+      != NULL)
 	readhistfile(fp);
     histsessionstart = lasthist;
     histsessionnew = 0;
@@ -1716,56 +1721,54 @@ read_hist()
 #endif
 
 static void
-col_0()
-{
+col_0(void) {
     linelim = 0;
 }
 
 static void
-last_col()
-{
+last_col(void) {
     linelim = strlen(line);
     if (linelim > 0 && mode_ind == 'e')
 	--linelim;
 }
 
 static int
-find_char(int arg, int n)
-{
-    register int i;
+find_char(int a, int n) {
+	register int i;
 
-    if (findchar)
-	finddir = n;
-    findchar = vigetch();
-    switch (dotb[doti - 1]) {
-	case 'f': case 'F': case 't': case 'T':
-	    savedot(findchar);
-	default:
-	    break;
-    }
-    i = linelim;
-    while (arg--) {
-	i += n;
-	while (i >= 0 && line[i] && line[i] != findchar)
-	    i += n;
-	if (i < 0 || !line[i]) {
-	    i = linelim;
-	    break;
+	if (findchar)
+		finddir = n;
+	findchar = vigetch();
+	if (doti > 0)
+		switch (dotb[doti - 1]) {
+		case 'f': case 'F': case 't': case 'T':
+			savedot(findchar);
+		default:
+			break;
+		}
+	i = linelim;
+	while (a--) {
+		i += n;
+		while (i >= 0 && line[i] && line[i] != findchar)
+			i += n;
+		if (i < 0 || !line[i]) {
+			i = linelim;
+			break;
+		}
 	}
-    }
-    findfunc = 'f';
-    return (i);
+	findfunc = 'f';
+	return i;
 }
 
 static int
-to_char(int arg, int n)
+to_char(int a, int n)
 {
     register int i;
     int tmp = linelim;
 
-    if (linelim + n >= 0 && linelim + n < strlen(line))
+    if (linelim + n >= 0 && (size_t)(linelim + n) < strlen(line))
 	linelim += n;
-    i = find_char(arg, n);
+    i = find_char(a, n);
     if (i != linelim)
 	i -= n;
     linelim = tmp;
@@ -1775,9 +1778,7 @@ to_char(int arg, int n)
 }
 
 static void
-match_paren()
-{
-    register int i;
+match_paren(void) {
     int nest = 1;
     int tmp = linelim;
 
@@ -1827,8 +1828,7 @@ remember(int save)
 }
 
 void
-gohome()
-{
+gohome(void) {
     struct frange *fr;
 
     remember(0);
@@ -1860,8 +1860,7 @@ gohome()
 }
 
 void
-leftlimit()
-{
+leftlimit(void) {
     struct frange *fr;
 
     remember(0);
@@ -1884,8 +1883,7 @@ leftlimit()
 }
 
 void
-rightlimit()
-{
+rightlimit(void) {
     register struct ent *p;
     struct frange *fr;
 
@@ -1920,8 +1918,7 @@ rightlimit()
 }
 
 void
-gototop()
-{
+gototop(void) {
     struct frange *fr;
 
     remember(0);
@@ -1944,8 +1941,7 @@ gototop()
 }
 
 void
-gotobottom()
-{
+gotobottom(void) {
     register struct ent *p;
     struct frange *fr;
 
@@ -1980,8 +1976,7 @@ gotobottom()
 }
 
 static void
-dogoto()
-{
+dogoto(void) {
     static char		*tempc = NULL;
     static unsigned	templen = 0;
     int			tempi;
@@ -1991,7 +1986,7 @@ dogoto()
 	tempc = scxrealloc(tempc, templen);
     }
 
-    strcpy(tempc, line);
+    strlcpy(tempc, line, templen);
     tempi = linelim;
 
     /* Can't switch back to navigate mode if insert_mode() is used here
@@ -2005,12 +2000,12 @@ dogoto()
     query("goto where?", NULL);
     if (linelim >= 0) {
 	memmove(line + 5, line, strlen(line) + 1);
-	strncpy(line, "goto ", 5);
+	strlcpy(line, "goto ", 5);
 	linelim = 0;
 	yyparse();
     }
 
-    strcpy(line, tempc);
+    strlcpy(line, tempc, sizeof line);
     linelim = tempi;
     /* Now we need to change back to navigate mode ourselves so that
      * toggle_navigate_mode() will work properly again.
@@ -2022,30 +2017,32 @@ dogoto()
 }
 
 void
-query(char *s, char *data)
+query(const char *s, char *data)
 {
     int c;
 
     insert_mode();
     if (data != NULL) {
-	strcpy(line, data);
+	strlcpy(line, data, sizeof line);
 	linelim = strlen(line);
     } else {
     	*line = '\0';
 	linelim = 0;
     }
-    if (s != NULL) error(s);
+    if (s != NULL) {
+    	error(s);
+    }
 
     while (linelim >= 0) {
 	update(0);
 	switch (c = nmgetch()) {
 	    case ctl('m'):
-		error("");
+		CLEAR_LINE;
 		return;
 	    case ctl('g'):
 		line[0] = '\0';
 		linelim = -1;
-		error("");
+		CLEAR_LINE;
 		update(0);
 		return;
 	    case ctl('l'):
@@ -2058,3 +2055,20 @@ query(char *s, char *data)
 	}
     }
 }
+
+#ifdef NCURSES_MOUSE_VERSION
+void
+mouse_set_pos(int m) {
+	static int x0;
+	switch (m) {
+	case 1:
+		x0 = mevent.x;
+		break;
+	case 2:
+		if (x0 != mevent.x)
+			break;
+	default:
+		linelim = mevent.x - 3;
+	}
+}
+#endif
