@@ -3,25 +3,19 @@
  * $Revision: 7.16 $
  */
 
-#include <curses.h>
+#include <stdlib.h>
+#include <limits.h>
+#include "compat.h"
 #include "sc.h"
 
-extern char	*malloc();
-extern char	*realloc();
-extern void	free();
-void		fatal();
-
-#ifdef SYSV3
-extern void	free();
-extern void	exit();
-#endif
+static void fatal(char *);
 
 #define	MAGIC	(double)1234567890.12344
 
-char *
-scxmalloc(unsigned n)
+void *
+scxmalloc(size_t n)
 {
-	register char *ptr;
+	void *ptr;
 
 	if ((ptr = malloc(n + sizeof(double))) == NULL)
 		fatal("scxmalloc: no memory");
@@ -30,8 +24,8 @@ scxmalloc(unsigned n)
 }
 
 /* we make sure realloc will do a malloc if needed */
-char *
-scxrealloc(char *ptr, unsigned n)
+void *
+scxrealloc(void *ptr, size_t n)
 {
 	if (ptr == NULL)
 		return(scxmalloc(n));
@@ -47,7 +41,7 @@ scxrealloc(char *ptr, unsigned n)
 }
 
 void
-scxfree(char *p)
+scxfree(void *p)
 {
 	if (p == NULL)
 		fatal("scxfree: NULL");
@@ -57,20 +51,15 @@ scxfree(char *p)
 	free(p);
 }
 
-#ifdef PSC
-void
-fatal(char *str)
-{
-    (void) fprintf(stderr,"%s\n", str);
-    exit(1);
-}
-#else
-void
+static void
 fatal(char *str)
 {
+#ifndef PSC
     deraw(1);
-    (void) fprintf(stderr,"%s\n", str);
+#endif /* PSC */
+    fprintf(stderr, "%s\n", str);
+#ifndef PSC
     diesave();
+#endif /* PSC */
     exit(1);
 }
-#endif /* PSC */
